@@ -27,6 +27,12 @@
  * Contributor(s): ______________________________________.
  *
  * $Log: codecs.h,v $
+ * Revision 1.70  2003/12/14 10:42:29  rjongbloed
+ * Changes for compilability without video support.
+ *
+ * Revision 1.69  2003/11/12 11:14:21  csoutheren
+ * Added H323FramedAudioCodec::DecodeSilenceFrame thanks to Henry Harrison of AliceStreet
+ *
  * Revision 1.68  2003/07/16 10:43:12  csoutheren
  * Added SwapChannel function to H323Codec to allow media hold channels
  * to work better. Thanks to Federico Pinna
@@ -266,6 +272,8 @@
 #include <mediafmt.h>
 #include <rtp.h>
 #include <channels.h>
+#include "openh323buildopts.h"
+
 
 /* The following classes have forward references to avoid including the VERY
    large header files for H225 and H245. If an application requires access
@@ -716,6 +724,15 @@ class H323FramedAudioCodec : public H323AudioCodec
       unsigned & written    /// Number of bytes used from data buffer
     ) = 0;
 
+    /**
+      Called when a frame is missed due to late arrival or other reasons
+      By default, this function fills the buffer with silence
+      */
+    virtual void DecodeSilenceFrame(
+      void * buffer,  /// Buffer from which encoded data is found
+      unsigned length       /// Length of encoded data buffer
+    )
+    { memset(buffer, 0, length); }
 
   protected:
     PShortArray sampleBuffer;
