@@ -27,6 +27,9 @@
  * Contributor(s): ______________________________________.
  *
  * $Log: gkclient.h,v $
+ * Revision 1.52  2004/03/04 04:49:34  csoutheren
+ * Added access function for gatekeeper routed address
+ *
  * Revision 1.51  2003/03/26 00:46:25  robertj
  * Had another go at making H323Transactor being able to be created
  *   without having a listener running.
@@ -432,6 +435,12 @@ class H323Gatekeeper : public H225_RAS
       const PString & password,            /// New password
       const PString & username = PString() /// Username for password
     );
+
+    /*
+     * Return the call signalling address for the gatekeeper (if present)
+     */
+    H323TransportAddress GetGatekeeperRouteAddress() const
+    { return gkRouteAddress; }
   //@}
 	
 
