@@ -27,6 +27,28 @@
  * Contributor(s): ______________________________________.
  *
  * $Log: h323.h,v $
+ * Revision 1.132  2003/06/06 02:19:03  rjongbloed
+ * Added iLBC codec
+ *
+ * Revision 1.131  2003/05/14 03:29:56  rjongbloed
+ * Added H.263 codec and autoconf options header file.
+ *
+ * Revision 1.130  2002/11/05 02:18:03  robertj
+ * Changed default for speex codec to be included.
+ *
+ * Revision 1.129  2002/11/04 02:57:24  dereks
+ * Put test around include of speexcodec.h    SPEEX is now an optional codec.
+ *
+ * Revision 1.128  2002/10/24 05:34:03  robertj
+ * Added speex codec
+ *
+ * Revision 1.127  2002/10/01 03:07:12  robertj
+ * Added version number functions for OpenH323 library itself, plus included
+ *   library version in the default vendor information.
+ *
+ * Revision 1.126  2002/06/27 03:08:57  robertj
+ * Added code to include G.729 & G.726 codecs on static linking.
+ *
  * Revision 1.125  2001/10/23 02:18:06  dereks
  * Initial release of CU30 video codec.
  *
@@ -428,6 +450,8 @@
  *
  */
 
+#include "openh323buildopts.h"
+
 #include "h323con.h"
 #include "h323ep.h"
 #include "gkclient.h"
@@ -436,14 +460,27 @@
 #include "gsmcodec.h"
 #include "mscodecs.h"
 #include "lpc10codec.h"
+#include "g729codec.h"
+#include "g726codec.h"
+#ifndef NO_SPEEX
+#include "speexcodec.h"
+#endif
+#include "ilbccodec.h"
 #endif // NO_H323_AUDIO_CODECS
 
 #ifndef NO_H323_VIDEO
 #include "h261codec.h"
+#include "h263codec.h"
 #ifdef HAS_CU30
 #include "cu30codec.h"
 #endif
 #endif // NO_H323_VIDEO
 
 
+PString  OpalGetVersion();
+unsigned OpalGetMajorVersion();
+unsigned OpalGetMinorVersion();
+unsigned OpalGetBuildNumber();
+
+
 /////////////////////////////////////////////////////////////////////////////
