@@ -27,6 +27,13 @@
  * Contributor(s): ______________________________________.
  *
  * $Log: h323con.h,v $
+ * Revision 1.66  2004/03/04 04:48:29  csoutheren
+ * Added virtual keyword to allow overring of capability management functions
+ * Added additional response code needed for MCU
+ *
+ * Revision 1.65  2004/02/26 08:32:47  csoutheren
+ * Added release complete codes for MCU
+ *
  * Revision 1.64  2003/12/28 02:38:14  csoutheren
  * Added H323EndPoint::OnOutgoingCall
  *
@@ -459,6 +466,7 @@ class H323Connection : public PObject
       EndedByTemporaryFailure,  /// The remote failed temporarily app may retry
       EndedByQ931Cause,         /// The remote ended the call with unmapped Q.931 cause code
       EndedByDurationLimit,     /// Call cleared due to an enforced duration limit
+      EndedByInvalidConferenceID, // Call cleared due to invalid conference ID
       NumCallEndReasons
     };
 #if PTRACING
@@ -895,6 +903,7 @@ class H323Connection : public PObject
       AnswerCallDeferred,          /// As for AnswerCallPending but does not send Alerting PDU
       AnswerCallAlertWithMedia,    /// As for AnswerCallPending but starts media channels
       AnswerCallDeferredWithMedia, /// As for AnswerCallDeferred but starts media channels
+      AnswerCallDeniedByInvalidCID, /// As for AnswerCallDenied but returns e_invalidCID
       NumAnswerCallResponses
     };
 #if PTRACING
@@ -1131,7 +1140,7 @@ class H323Connection : public PObject
 
     /**Start control channel negotiations.
       */
-    BOOL StartControlNegotiations();
+    virtual BOOL StartControlNegotiations();
 
     /**Handle reading data on the control channel.
      */
