@@ -24,6 +24,9 @@
  * Contributor(s): ______________________________________.
  *
  * $Log: mediafmt.h,v $
+ * Revision 1.14  2004/02/26 08:19:31  csoutheren
+ * Fixed threading problem with GetMediaFormatList
+ *
  * Revision 1.13  2002/12/02 03:06:26  robertj
  * Fixed over zealous removal of code when NO_AUDIO_CODECS set.
  *
@@ -188,11 +191,9 @@ class OpalMediaFormat : public PCaselessString
       VideoTimeUnits = 90  // 90kHz sample rate
     };
 
-
     /**Get the list of media formats that have been registered.
       */
-    inline static const List & GetRegisteredMediaFormats() { return GetMediaFormatsList(); }
-
+    static List GetRegisteredMediaFormats();
 
   protected:
     RTP_DataFrame::PayloadTypes rtpPayloadType;
@@ -203,7 +204,9 @@ class OpalMediaFormat : public PCaselessString
     unsigned frameTime;
     unsigned timeUnits;
 
+  private:
     static List & GetMediaFormatsList();
+    static PMutex & GetMediaFormatsListMutex();
 };
 
 
