@@ -24,6 +24,49 @@
  * Contributor(s): ______________________________________.
  *
  * $Log: q931.h,v $
+ * Revision 1.52  2003/03/18 05:54:14  robertj
+ * Added ChannelIdentifier IE support, thanks Eize Slange
+ *
+ * Revision 1.51  2003/02/12 00:02:06  robertj
+ * Added more Q.931 cause codes.
+ * Added ability to trace text version of cause codes and IE codes.
+ *
+ * Revision 1.50  2002/11/25 22:40:00  robertj
+ * Added another Q.850 code
+ *
+ * Revision 1.49  2002/11/07 03:49:49  robertj
+ * Added extra "congested" Q.931 codes.
+ *
+ * Revision 1.48  2002/09/16 01:14:15  robertj
+ * Added #define so can select if #pragma interface/implementation is used on
+ *   platform basis (eg MacOS) rather than compiler, thanks Robert Monaghan.
+ *
+ * Revision 1.47  2002/09/03 05:42:27  robertj
+ * Normalised the multi-include header prevention ifdef/define symbol.
+ * Removed redundent includes.
+ *
+ * Revision 1.46  2002/08/05 10:03:47  robertj
+ * Cosmetic changes to normalise the usage of pragma interface/implementation.
+ *
+ * Revision 1.45  2002/07/05 03:13:35  robertj
+ * Added copy constructor for Q.931 so makes duplicate instead of reference to IE's.
+ *
+ * Revision 1.44  2002/05/22 23:12:01  robertj
+ * Enhanced the display of Release-Complete cause codes.
+ *
+ * Revision 1.43  2002/05/03 05:38:16  robertj
+ * Added Q.931 Keypad IE mechanism for user indications (DTMF).
+ *
+ * Revision 1.42  2002/04/22 08:03:41  craigs
+ * Added Q931 progress codes
+ *
+ * Revision 1.41  2002/04/22 07:33:02  craigs
+ * Changed GetProgressIndicator to be const
+ * Fixed spelling mistake in SignalType enums list
+ *
+ * Revision 1.40  2002/04/18 09:35:00  robertj
+ * Added CallState IE processing.
+ *
  * Revision 1.39  2002/03/27 06:04:42  robertj
  * Added Temporary Failure end code for connection, an application may
  *   immediately retry the call if this occurs.
@@ -150,15 +193,10 @@
  *
  */
 
-#ifndef __Q931_H
-#define __Q931_H
+#ifndef __OPAL_Q931_H
+#define __OPAL_Q931_H
 
-
-#include <ptlib/sockets.h>
-#include "h225.h"
-
-
-#ifdef __GNUC__
+#ifdef P_USE_PRAGMA
 #pragma interface
 #endif
 
@@ -202,6 +240,7 @@ class Q931 : public PObject
     };
 
     Q931();
+    Q931(const Q931 & other);
     Q931 & operator=(const Q931 & other);
 
     void BuildFacility(int callRef, BOOL fromDest);
@@ -235,19 +274,23 @@ class Q931 : public PObject
     MsgTypes GetMessageType() const { return messageType; }
 
     enum InformationElementCodes {
-      BearerCapabilityIE   = 0x04,
-      CauseIE              = 0x08,
-      FacilityIE           = 0x1c,
-      ProgressIndicatorIE  = 0x1e,
-      CallStateIE          = 0x14,
-      DisplayIE            = 0x28,
-      SignalIE             = 0x34,
-      ConnectedNumberIE    = 0x4c,
-      CallingPartyNumberIE = 0x6c,
-      CalledPartyNumberIE  = 0x70,
-      RedirectingNumberIE  = 0x74,
-      UserUserIE           = 0x7e
+      BearerCapabilityIE      = 0x04,
+      CauseIE                 = 0x08,
+      ChannelIdentificationIE = 0x18,
+      FacilityIE              = 0x1c,
+      ProgressIndicatorIE     = 0x1e,
+      CallStateIE             = 0x14,
+      DisplayIE               = 0x28,
+      KeypadIE                = 0x2c,
+      SignalIE                = 0x34,
+      ConnectedNumberIE       = 0x4c,
+      CallingPartyNumberIE    = 0x6c,
+      CalledPartyNumberIE     = 0x70,
+      RedirectingNumberIE     = 0x74,
+      UserUserIE              = 0x7e
     };
+    friend ostream & operator<<(ostream & strm, InformationElementCodes ie);
+
     BOOL HasIE(InformationElementCodes ie) const;
     PBYTEArray GetIE(InformationElementCodes ie) const;
     void SetIE(InformationElementCodes ie, const PBYTEArray & userData);
@@ -277,27 +320,46 @@ class Q931 : public PObject
     );
 
     enum CauseValues {
-      UnknownCauseIE        = 0x00,
-      NoRouteToNetwork      = 0x02,
-      NoRouteToDestination  = 0x03,
-      ChannelUnacceptable   = 0x06,
-      NormalCallClearing    = 0x10,
-      UserBusy              = 0x11,
-      NoResponse            = 0x12,
-      NoAnswer              = 0x13,
-      SubscriberAbsent      = 0x14,
-      CallRejected          = 0x15,
-      NumberChanged         = 0x16,
-      Redirection           = 0x17,
-      DestinationOutOfOrder = 0x1b,
-      InvalidNumberFormat   = 0x1c,
-      StatusEnquiryResponse = 0x1e,
-      NoCircuitChannelAvailable = 0x22,
-      TemporaryFailure      = 0x29,
-      Congestion            = 0x2a,
-      InvalidCallReference  = 0x51,
-      ErrorInCauseIE        = 0x100
+      UnknownCauseIE               =  0,
+      UnallocatedNumber            =  1,
+      NoRouteToNetwork             =  2,
+      NoRouteToDestination         =  3,
+      SendSpecialTone              =  4,
+      MisdialledTrunkPrefix        =  5,
+      ChannelUnacceptable          =  6,
+      NormalCallClearing           = 16,
+      UserBusy                     = 17,
+      NoResponse                   = 18,
+      NoAnswer                     = 19,
+      SubscriberAbsent             = 20,
+      CallRejected                 = 21,
+      NumberChanged                = 22,
+      Redirection                  = 23,
+      ExchangeRoutingError         = 25,
+      NonSelectedUserClearing      = 26,
+      DestinationOutOfOrder        = 27,
+      InvalidNumberFormat          = 28,
+      FacilityRejected             = 29,
+      StatusEnquiryResponse        = 30,
+      NormalUnspecified            = 31,
+      NoCircuitChannelAvailable    = 34,
+      NetworkOutOfOrder            = 38,
+      TemporaryFailure             = 41,
+      Congestion                   = 42,
+      RequestedCircuitNotAvailable = 44,
+      ResourceUnavailable          = 47,
+      ServiceOptionNotAvailable    = 63,
+      InvalidCallReference         = 81,
+      ClearedRequestedCallIdentity = 86,
+      IncompatibleDestination      = 88,
+      IENonExistantOrNotImplemented= 99,
+      TimerExpiry                  = 102,
+      ProtocolErrorUnspecified     = 111,
+      InterworkingUnspecified      = 127,
+      ErrorInCauseIE               = 0x100
     };
+    friend ostream & operator<<(ostream & strm, CauseValues cause);
+
     void SetCause(
       CauseValues value,
       unsigned standard = 0,  // 0 = ITU-T standardized coding
@@ -308,6 +370,33 @@ class Q931 : public PObject
       unsigned * location = NULL   // 0 = User
     ) const;
 
+    enum CallStates {
+      CallState_Null                  = 0,
+      CallState_CallInitiated         = 1,
+      CallState_OverlapSending        = 2,
+      CallState_OutgoingCallProceeding= 3,
+      CallState_CallDelivered         = 4,
+      CallState_CallPresent           = 6,
+      CallState_CallReceived          = 7,
+      CallState_ConnectRequest        = 8,
+      CallState_IncomingCallProceeding= 9,
+      CallState_Active                = 10,
+      CallState_DisconnectRequest     = 11,
+      CallState_DisconnectIndication  = 12,
+      CallState_SuspendRequest        = 15,
+      CallState_ResumeRequest         = 17,
+      CallState_ReleaseRequest        = 19,
+      CallState_OverlapReceiving      = 25,
+      CallState_ErrorInIE             = 0x100
+    };
+    void SetCallState(
+      CallStates value,
+      unsigned standard = 0  // 0 = ITU-T standardized coding
+    );
+    CallStates GetCallState(
+      unsigned * standard = NULL  // 0 = ITU-T standardized coding
+    ) const;
+
     enum SignalInfo {
       SignalDialToneOn,
       SignalRingBackToneOn,
@@ -328,12 +417,26 @@ class Q931 : public PObject
       SignalAlertingPattern5,
       SignalAlertingPattern6,
       SignalAlertingPattern7,
-      SignalAlretingOff = 0x4f,
+      SignalAlertingOff = 0x4f,
       SignalErrorInIE = 0x100
     };
     void SetSignalInfo(SignalInfo value);
     SignalInfo GetSignalInfo() const;
 
+    void SetKeypad(const PString & digits);
+    PString GetKeypad() const;
+
+    enum ProgressIndication {
+       ProgressNotEndToEndISDN      = 1,      // Call is not end-to-end ISDN; 
+                                              // further call progress information may be available in-band  
+       ProgressDestinationNonISDN   = 2,      // Destination address is non ISDN  
+       ProgressOriginNotISDN        = 3,      // Origination address is non ISDN  
+       ProgressReturnedToISDN       = 4,      // Call has returned to the ISDN 
+       ProgressServiceChange        = 5,      // Interworking has occurred and has 
+                                              // resulted in a telecommunication service change
+       ProgressInbandInformationAvailable = 8 // In-band information or an appropriate pattern is now available.   
+    };
+
     void SetProgressIndicator(
       unsigned description,
       unsigned codingStandard = 0,
@@ -343,7 +446,7 @@ class Q931 : public PObject
       unsigned & description,
       unsigned * codingStandard = NULL,
       unsigned * location = NULL
-    );
+    ) const;
 
     void SetDisplayName(const PString & name);
     PString GetDisplayName() const;
@@ -436,6 +539,26 @@ class Q931 : public PObject
       unsigned defReason =0           // Default value if octet 3b not present
     ) const;
 
+    /**Set the limitations to ChannelIdentification.
+        - the interface identifier cannot be specified
+        - channel in PRI can only be indicated by number and cannot be indicated by map
+        - one and only one channel can be indicated
+        - the coding standard is always ITU Q.931
+      */
+    void SetChannelIdentification(
+      unsigned interfaceType = 0,        //  0 = basic,     1 = other (e.g. primary)
+      unsigned preferredOrExclusive = 0, //  0 = preferred, 1 = exclusive
+      int      channelNumber = 1         // -1 = any,       0 = none/D, 1 = channel 1/B1, etc. 1-15,17-31
+    );
+
+    /**Get the limitations to ChannelIdentification.
+      */
+    BOOL GetChannelIdentification(
+      unsigned * interfaceType = NULL,        // Interface type
+      unsigned * preferredOrExclusive = NULL, // Channel negotiation preference
+      int      * channelNumber = NULL         // Channel number
+    ) const;
+
   protected:
     unsigned callReference;
     BOOL fromDestination;
@@ -447,7 +570,7 @@ class Q931 : public PObject
 };
 
 
-#endif // __Q931_H
+#endif // __OPAL_Q931_H
 
 
 /////////////////////////////////////////////////////////////////////////////
