@@ -27,6 +27,65 @@
  * Contributor(s): ______________________________________.
  *
  * $Log: rtp.h,v $
+ * Revision 1.51  2003/10/27 06:03:39  csoutheren
+ * Added support for QoS
+ *   Thanks to Henry Harrison of AliceStreet
+ *
+ * Revision 1.50  2003/10/09 09:47:45  csoutheren
+ * Fixed problem with re-opening RTP half-channels under unusual
+ * circumstances. Thanks to Damien Sandras
+ *
+ * Revision 1.49  2003/05/14 13:46:39  rjongbloed
+ * Removed hack of using special payload type for H.263 for a method which
+ *   would be less prone to failure in the future.
+ *
+ * Revision 1.48  2003/05/02 04:57:43  robertj
+ * Added header extension support to RTP data frame class.
+ *
+ * Revision 1.47  2003/05/02 04:21:53  craigs
+ * Added hacked OpalH263 codec
+ *
+ * Revision 1.46  2003/02/07 00:30:41  robertj
+ * Changes for bizarre usage of RTP code outside of scope of H.323 specs.
+ *
+ * Revision 1.45  2003/02/05 06:32:08  robertj
+ * Fixed non-stun symmetric NAT support recently broken.
+ *
+ * Revision 1.44  2003/02/04 07:06:41  robertj
+ * Added STUN support.
+ *
+ * Revision 1.43  2002/11/19 01:48:15  robertj
+ * Allowed get/set of canonical anme and tool name.
+ *
+ * Revision 1.42  2002/11/12 22:10:48  robertj
+ * Updated documentation.
+ *
+ * Revision 1.41  2002/10/31 00:33:29  robertj
+ * Enhanced jitter buffer system so operates dynamically between minimum and
+ *   maximum values. Altered API to assure app writers note the change!
+ *
+ * Revision 1.40  2002/09/26 04:01:58  robertj
+ * Fixed calculation of fraction of packets lost in RR, thanks Awais Ali
+ *
+ * Revision 1.39  2002/09/16 01:14:15  robertj
+ * Added #define so can select if #pragma interface/implementation is used on
+ *   platform basis (eg MacOS) rather than compiler, thanks Robert Monaghan.
+ *
+ * Revision 1.38  2002/09/03 05:47:02  robertj
+ * Normalised the multi-include header prevention ifdef/define symbol.
+ * Added copy constructor/operator for session manager.
+ *
+ * Revision 1.37  2002/08/05 10:03:47  robertj
+ * Cosmetic changes to normalise the usage of pragma interface/implementation.
+ *
+ * Revision 1.36  2002/05/28 02:37:37  robertj
+ * Fixed reading data out of RTCP compound statements.
+ *
+ * Revision 1.35  2002/05/02 05:58:24  robertj
+ * Changed the mechanism for sending RTCP reports so that they will continue
+ *   to be sent regardless of if there is any actual data traffic.
+ * Added support for compound RTCP statements for sender and receiver reports.
+ *
  * Revision 1.34  2002/02/09 02:33:37  robertj
  * Improved payload type docuemntation and added Cisco CN.
  *
@@ -138,20 +197,33 @@
  *
  */
 
-#ifndef __RTP_H
-#define __RTP_H
+#ifndef __OPAL_RTP_H
+#define __OPAL_RTP_H
+
+#ifdef P_USE_PRAGMA
+#pragma interface
+#endif
 
 
 #include <ptlib/sockets.h>
 
 
 class RTP_JitterBuffer;
+class PSTUNClient;
 
 
-#ifdef __GNUC__
-#pragma interface
-#endif
 
+///////////////////////////////////////////////////////////////////////////////
+// 
+// class to hold the QoS definitions for an RTP channel
+
+class RTP_QOS : public PObject
+{
+  PCLASSINFO(RTP_QOS,PObject);
+  public:
+    PQoS dataQoS;
+    PQoS ctrlQoS;
+};
 
 ///////////////////////////////////////////////////////////////////////////////
 // Real Time Protocol - IETF RFC1889 and RFC1890
@@ -201,6 +273,7 @@ class RTP_DataFrame : public PBYTEArray
       H263,         // H.263
 
       LastKnownPayloadType,
+
       DynamicBase = 96,
       MaxPayloadType = 127,
       IllegalPayloadType
@@ -230,7 +303,13 @@ class RTP_DataFrame : public PBYTEArray
     DWORD  GetContribSource(PINDEX idx) const;
     void   SetContribSource(PINDEX idx, DWORD src);
 
-    PINDEX GetHeaderSize()     const { return 12 + 4*GetContribSrcCount(); }
+    PINDEX GetHeaderSize() const;
+
+    int GetExtensionType() const; // -1 is no extension
+    void   SetExtensionType(int type);
+    PINDEX GetExtensionSize() const;
+    BOOL   SetExtensionSize(PINDEX sz);
+    BYTE * GetExtensionPtr() const;
 
     PINDEX GetPayloadSize() const { return payloadSize; }
     BOOL   SetPayloadSize(PINDEX sz);
@@ -244,6 +323,8 @@ class RTP_DataFrame : public PBYTEArray
 #endif
 };
 
+PLIST(RTP_DataFrameList, RTP_DataFrame);
+
 
 /**An RTP control frame encapsulation.
   */
@@ -252,11 +333,11 @@ class RTP_ControlFrame : public PBYTEArray
   PCLASSINFO(RTP_ControlFrame, PBYTEArray);
 
   public:
-    RTP_ControlFrame(PINDEX payloadSize = 2048);
+    RTP_ControlFrame(PINDEX compoundSize = 2048);
 
-    unsigned GetVersion() const { return (BYTE)theArray[0]>>6; }
+    unsigned GetVersion() const { return (BYTE)theArray[compoundOffset]>>6; }
 
-    unsigned GetCount() const { return (BYTE)theArray[0]&0x1f; }
+    unsigned GetCount() const { return (BYTE)theArray[compoundOffset]&0x1f; }
     void     SetCount(unsigned count);
 
     enum PayloadTypes {
@@ -267,13 +348,18 @@ class RTP_ControlFrame : public PBYTEArray
       e_ApplDefined
     };
 
-    unsigned GetPayloadType() const { return (BYTE)theArray[1]; }
+    unsigned GetPayloadType() const { return (BYTE)theArray[compoundOffset+1]; }
     void     SetPayloadType(unsigned t);
 
-    PINDEX GetPayloadSize() const { return 4*(*(PUInt16b *)&theArray[2]); }
+    PINDEX GetPayloadSize() const { return 4*(*(PUInt16b *)&theArray[compoundOffset+2]); }
     void   SetPayloadSize(PINDEX sz);
 
-    BYTE * GetPayloadPtr() const { return (BYTE *)(theArray+4); }
+    BYTE * GetPayloadPtr() const { return (BYTE *)(theArray+compoundOffset+4); }
+
+    BOOL ReadNextCompound();
+    BOOL WriteNextCompound();
+
+    PINDEX GetCompoundSize() const { return compoundSize; }
 
 #pragma pack(1)
     struct ReceiverReport {
@@ -333,6 +419,10 @@ class RTP_ControlFrame : public PBYTEArray
       const PString & data      /// Data for description
     );
 #pragma pack()
+
+  protected:
+    PINDEX compoundOffset;
+    PINDEX compoundSize;
 };
 
 
@@ -348,7 +438,7 @@ class RTP_UserData : public PObject
 
   public:
     /**Callback from the RTP session for transmit statistics monitoring.
-       This is called every RTP_Session::senderReportInterval packets on the
+       This is called every RTP_Session::txStatisticsInterval packets on the
        transmitter indicating that the statistics have been updated.
 
        The default behaviour does nothing.
@@ -409,10 +499,22 @@ class RTP_Session : public PObject
        attached to this RTP session.
       */
     void SetJitterBufferSize(
-      unsigned jitterDelay,    /// Total jitter buffer delay in milliseconds
+      unsigned minJitterDelay, /// Minimum jitter buffer delay in RTP timestamp units
+      unsigned maxJitterDelay, /// Maximum jitter buffer delay in RTP timestamp units
       PINDEX stackSize = 30000 /// Stack size for jitter thread
     );
 
+    /**Get current size of the jitter buffer.
+       This returns the currently used jitter buffer delay in RTP timestamp
+       units. It will be some value between the minimum and maximum set in
+       the SetJitterBufferSize() function.
+      */
+    unsigned GetJitterBufferSize() const;
+
+    /**Modifies the QOS specifications for this RTP session*/
+    virtual BOOL ModifyQOS(RTP_QOS * )
+    { return FALSE; }
+
     /**Read a data frame from the RTP channel.
        This function will conditionally read data from eth jitter buffer or
        directly if there is no jitter buffer enabled. An application should
@@ -444,6 +546,10 @@ class RTP_Session : public PObject
       RTP_ControlFrame & frame    /// Frame to write to the RTP session
     ) = 0;
 
+    /**Write the RTCP reports.
+      */
+    virtual BOOL SendReport();
+
     /**Close down the RTP session.
       */
     virtual void Close(
@@ -455,7 +561,7 @@ class RTP_Session : public PObject
     virtual PString GetLocalHostName() = 0;
   //@}
 
-  /**@name Call back frunctions */
+  /**@name Call back functions */
   //@{
     enum SendReceiveStatus {
       e_ProcessPacket,
@@ -464,7 +570,7 @@ class RTP_Session : public PObject
     };
     virtual SendReceiveStatus OnSendData(RTP_DataFrame & frame);
     virtual SendReceiveStatus OnReceiveData(const RTP_DataFrame & frame);
-    virtual SendReceiveStatus OnReceiveControl(const RTP_ControlFrame & frame);
+    virtual SendReceiveStatus OnReceiveControl(RTP_ControlFrame & frame);
 
     class ReceiverReport : public PObject  {
         PCLASSINFO(ReceiverReport, PObject);
@@ -522,6 +628,22 @@ class RTP_Session : public PObject
       */
     unsigned GetSessionID() const { return sessionID; }
 
+    /**Get the canonical name for the RTP session.
+      */
+    PString GetCanonicalName() const;
+
+    /**Set the canonical name for the RTP session.
+      */
+    void SetCanonicalName(const PString & name);
+
+    /**Get the tool name for the RTP session.
+      */
+    PString GetToolName() const;
+
+    /**Set the tool name for the RTP session.
+      */
+    void SetToolName(const PString & name);
+
     /**Get the user data for the session.
       */
     RTP_UserData * GetUserData() const { return userData; }
@@ -564,24 +686,34 @@ class RTP_Session : public PObject
       BOOL ignore   /// Flag for ignore out of order packets
     ) { ignoreOutOfOrderPackets = ignore; }
 
-    /**Get the interval for transmitter reports in the session.
+    /**Get the time interval for sending RTCP reports in the session.
       */
-    unsigned GetSenderReportInterval() { return senderReportInterval; }
+    const PTimeInterval & GetReportTimeInterval() { return reportTimeInterval; }
 
-    /**Set the interval for transmitter reports in the session.
+    /**Set the time interval for sending RTCP reports in the session.
       */
-    void SetSenderReportInterval(
-      unsigned packets   // Number of packets between reports
+    void SetReportTimeInterval(
+      const PTimeInterval & interval /// New time interval for reports.
+    )  { reportTimeInterval = interval; }
+
+    /**Get the interval for transmitter statistics in the session.
+      */
+    unsigned GetTxStatisticsInterval() { return txStatisticsInterval; }
+
+    /**Set the interval for transmitter statistics in the session.
+      */
+    void SetTxStatisticsInterval(
+      unsigned packets   /// Number of packets between callbacks
     );
 
-    /**Get the interval for receiver reports in the session.
+    /**Get the interval for receiver statistics in the session.
       */
-    unsigned GetReceiverReportInterval() { return receiverReportInterval; }
+    unsigned GetRxStatisticsInterval() { return rxStatisticsInterval; }
 
-    /**Set the interval for receiver reports in the session.
+    /**Set the interval for receiver statistics in the session.
       */
-    void SetReceiverReportInterval(
-      unsigned packets   // Number of packets between reports
+    void SetRxStatisticsInterval(
+      unsigned packets   /// Number of packets between callbacks
     );
 
     /**Get total number of packets sent in session.
@@ -613,37 +745,37 @@ class RTP_Session : public PObject
     DWORD GetPacketsTooLate() const;
 
     /**Get average time between sent packets.
-       This is averaged over the last senderReportInterval packets and is in
+       This is averaged over the last txStatisticsInterval packets and is in
        milliseconds.
       */
     DWORD GetAverageSendTime() const { return averageSendTime; }
 
     /**Get maximum time between sent packets.
-       This is over the last senderReportInterval packets and is in
+       This is over the last txStatisticsInterval packets and is in
        milliseconds.
       */
     DWORD GetMaximumSendTime() const { return maximumSendTime; }
 
     /**Get minimum time between sent packets.
-       This is over the last senderReportInterval packets and is in
+       This is over the last txStatisticsInterval packets and is in
        milliseconds.
       */
     DWORD GetMinimumSendTime() const { return minimumSendTime; }
 
     /**Get average time between received packets.
-       This is averaged over the last receiverReportInterval packets and is in
+       This is averaged over the last rxStatisticsInterval packets and is in
        milliseconds.
       */
     DWORD GetAverageReceiveTime() const { return averageReceiveTime; }
 
     /**Get maximum time between received packets.
-       This is over the last receiverReportInterval packets and is in
+       This is over the last rxStatisticsInterval packets and is in
        milliseconds.
       */
     DWORD GetMaximumReceiveTime() const { return maximumReceiveTime; }
 
     /**Get minimum time between received packets.
-       This is over the last receiverReportInterval packets and is in
+       This is over the last rxStatisticsInterval packets and is in
        milliseconds.
       */
     DWORD GetMinimumReceiveTime() const { return minimumReceiveTime; }
@@ -652,13 +784,20 @@ class RTP_Session : public PObject
        This is the calculated statistical variance of the interarrival
        time of received packets in milliseconds.
       */
-    DWORD GetJitterTime() const { return jitterLevel>>7; }
+    DWORD GetAvgJitterTime() const { return jitterLevel>>7; }
+
+    /**Get averaged jitter time for received packets.
+       This is the maximum value of jitterLevel for the session.
+      */
+    DWORD GetMaxJitterTime() const { return maximumJitterLevel>>7; }
   //@}
 
   protected:
-    void SetReceiverReport(RTP_ControlFrame::ReceiverReport & receiver);
+    void AddReceiverReport(RTP_ControlFrame::ReceiverReport & receiver);
 
     unsigned           sessionID;
+    PString            canonicalName;
+    PString            toolName;
     unsigned           referenceCount;
     RTP_UserData     * userData;
     RTP_JitterBuffer * jitter;
@@ -667,10 +806,12 @@ class RTP_Session : public PObject
     BOOL          ignoreOutOfOrderPackets;
     DWORD         syncSourceOut;
     DWORD         syncSourceIn;
-    unsigned      senderReportInterval;
-    unsigned      receiverReportInterval;
+    PTimeInterval reportTimeInterval;
+    unsigned      txStatisticsInterval;
+    unsigned      rxStatisticsInterval;
     WORD          lastSentSequenceNumber;
     WORD          expectedSequenceNumber;
+    DWORD         lastSentTimestamp;
     PTimeInterval lastSentPacketTime;
     PTimeInterval lastReceivedPacketTime;
     WORD          lastRRSequenceNumber;
@@ -690,16 +831,22 @@ class RTP_Session : public PObject
     DWORD maximumReceiveTime;
     DWORD minimumReceiveTime;
     DWORD jitterLevel;
+    DWORD maximumJitterLevel;
+
+    unsigned txStatisticsCount;
+    unsigned rxStatisticsCount;
 
-    unsigned senderReportCount;
-    unsigned receiverReportCount;
     DWORD    averageSendTimeAccum;
     DWORD    maximumSendTimeAccum;
     DWORD    minimumSendTimeAccum;
     DWORD    averageReceiveTimeAccum;
     DWORD    maximumReceiveTimeAccum;
     DWORD    minimumReceiveTimeAccum;
+    DWORD    packetsLostSinceLastRR;
     DWORD    lastTransitTime;
+
+    PMutex reportMutex;
+    PTimer reportTimer;
 };
 
 
@@ -715,6 +862,8 @@ class RTP_SessionManager : public PObject
     /**Construct new session manager database.
       */
     RTP_SessionManager();
+    RTP_SessionManager(const RTP_SessionManager & sm);
+    RTP_SessionManager & operator=(const RTP_SessionManager & sm);
   //@}
 
 
@@ -853,6 +1002,10 @@ class RTP_UDP : public RTP_Session
     virtual PString GetLocalHostName();
   //@}
 
+    /**Change the QoS settings
+      */
+    virtual BOOL ModifyQOS(RTP_QOS * rtpqos);
+
   /**@name New functions for class */
   //@{
     /**Open the UDP ports for the RTP session.
@@ -861,16 +1014,29 @@ class RTP_UDP : public RTP_Session
       PIPSocket::Address localAddress,  /// Local interface to bind to
       WORD portBase,                    /// Base of ports to search
       WORD portMax,                     /// end of ports to search (inclusive)
-      BYTE ipTypeOfService              /// Type of Service byte
+      BYTE ipTypeOfService,             /// Type of Service byte
+      PSTUNClient * stun = NULL,        /// STUN server to use createing sockets (or NULL if no STUN)
+      RTP_QOS * rtpqos = NULL           /// QOS spec (or NULL if no QoS)
     );
   //@}
 
+   /**Reopens an existing session in the given direction.
+      */
+    void Reopen(BOOL isReading);
+  //@}
+
   /**@name Member variable access */
   //@{
     /**Get local address of session.
       */
     PIPSocket::Address GetLocalAddress() const { return localAddress; }
 
+    /**Set local address of session.
+      */
+    void SetLocalAddress(
+      const PIPSocket::Address & addr
+    ) { localAddress = addr; }
+
     /**Get remote address of session.
       */
     PIPSocket::Address GetRemoteAddress() const { return remoteAddress; }
@@ -893,11 +1059,11 @@ class RTP_UDP : public RTP_Session
 
     /**Get data UDP socket of session.
       */
-    PUDPSocket & GetDataSocket() { return dataSocket; }
+    PUDPSocket & GetDataSocket() { return *dataSocket; }
 
     /**Get control UDP socket of session.
       */
-    PUDPSocket & GetControlSocket() { return controlSocket; }
+    PUDPSocket & GetControlSocket() { return *controlSocket; }
 
     /**Set the remote address and port information for session.
       */
@@ -906,6 +1072,12 @@ class RTP_UDP : public RTP_Session
       WORD port,                    /// Port on remote
       BOOL isDataPort               /// Flag for data or control channel
     );
+
+    /**Apply QOS - requires address to connect the socket on Windows platforms
+     */
+    void ApplyQOS(
+      const PIPSocket::Address & addr
+    );
   //@}
 
   protected:
@@ -914,7 +1086,7 @@ class RTP_UDP : public RTP_Session
     SendReceiveStatus ReadDataOrControlPDU(
       PUDPSocket & socket,
       PBYTEArray & frame,
-      const char * PTRACE_name
+      BOOL fromDataChannel
     );
 
     PIPSocket::Address localAddress;
@@ -930,12 +1102,14 @@ class RTP_UDP : public RTP_Session
     BOOL shutdownRead;
     BOOL shutdownWrite;
 
-    PUDPSocket dataSocket;
-    PUDPSocket controlSocket;
+    PUDPSocket * dataSocket;
+    PUDPSocket * controlSocket;
+
+    BOOL appliedQOS;
 };
 
 
-#endif // __RTP_H
+#endif // __OPAL_RTP_H
 
 
 /////////////////////////////////////////////////////////////////////////////
