@@ -27,6 +27,51 @@
  * Contributor(s): ______________________________________.
  *
  * $Log: transports.h,v $
+ * Revision 1.43  2003/12/29 13:28:45  dominance
+ * fixed docbook syntax trying to generate LaTeX formula with ip$10.x.x.x.
+ *
+ * Revision 1.42  2003/04/10 09:44:55  robertj
+ * Added associated transport to new GetInterfaceAddresses() function so
+ *   interfaces can be ordered according to active transport links. Improves
+ *   interoperability.
+ * Replaced old listener GetTransportPDU() with GetInterfaceAddresses()
+ *   and H323SetTransportAddresses() functions.
+ *
+ * Revision 1.41  2003/04/10 01:03:25  craigs
+ * Added functions to access to lists of interfaces
+ *
+ * Revision 1.40  2003/03/21 05:24:02  robertj
+ * Added setting of remote port in UDP transport constructor.
+ *
+ * Revision 1.39  2003/02/06 04:29:23  robertj
+ * Added more support for adding things to H323TransportAddressArrays
+ *
+ * Revision 1.38  2002/11/21 06:39:56  robertj
+ * Changed promiscuous mode to be three way. Fixes race condition in gkserver
+ *   which can cause crashes or more PDUs to be sent to the wrong place.
+ *
+ * Revision 1.37  2002/11/10 08:10:43  robertj
+ * Moved constants for "well known" ports to better place (OPAL change).
+ *
+ * Revision 1.36  2002/09/16 01:14:15  robertj
+ * Added #define so can select if #pragma interface/implementation is used on
+ *   platform basis (eg MacOS) rather than compiler, thanks Robert Monaghan.
+ *
+ * Revision 1.35  2002/07/22 09:40:15  robertj
+ * Added ability to automatically convert string arrays, lists sorted lists
+ *   directly to H323TransportAddressArray.
+ *
+ * Revision 1.34  2002/07/02 10:02:29  robertj
+ * Added H323TransportAddress::GetIpAddress() so don't have to provide port
+ *   when you don't need it as in GetIpAndPort(),.
+ *
+ * Revision 1.33  2002/06/28 03:34:25  robertj
+ * Fixed issues with address translation on gatekeeper RAS channel.
+ *
+ * Revision 1.32  2002/06/12 03:51:59  robertj
+ * Added function to compare two transport addresses in a more intelligent
+ *   way that strict string comparison. Takes into account wildcarding.
+ *
  * Revision 1.31  2002/02/11 04:20:25  robertj
  * Fixed documentation errors, thanks Horacio J. Pea
  *
@@ -133,6 +178,10 @@
 #ifndef __TRANSPORTS_H
 #define __TRANSPORTS_H
 
+#ifdef P_USE_PRAGMA
+#pragma interface
+#endif
+
 
 #include <ptlib/sockets.h>
 
@@ -153,10 +202,6 @@ class H323Transport;
 class H323Gatekeeper;
 
 
-#ifdef __GNUC__
-#pragma interface
-#endif
-
 
 ///////////////////////////////////////////////////////////////////////////////
 
@@ -177,6 +222,19 @@ class H323TransportAddress : public PString
     BOOL SetPDU(H225_TransportAddress & pdu) const;
     BOOL SetPDU(H245_TransportAddress & pdu) const;
 
+    /**Determine if the two transport addresses are equivalent.
+      */
+    BOOL IsEquivalent(
+      const H323TransportAddress & address
+    );
+
+    /**Extract the ip address from transport address.
+       Returns FALSE, if the address is not an IP transport address.
+      */
+    BOOL GetIpAddress(
+      PIPSocket::Address & ip
+    ) const;
+
     /**Extract the ip address and port number from transport address.
        Returns FALSE, if the address is not an IP transport address.
       */
@@ -193,10 +251,10 @@ class H323TransportAddress : public PString
 
     /**Create a listener based on this transport address.
 
-       For example an address of "ip$10.0.0.1:1720" would create a TCP
+       For example an address of "#ip$10.0.0.1:1720#" would create a TCP
        listening socket that would be bound to the specific interface
        10.0.0.1 and listens on port 1720. Note that the address
-       "ip$*:1720" can be used to bind to INADDR_ANY.
+       "#ip$*:1720#" can be used to bind to INADDR_ANY.
 
        Also note that if the address has a trailing '+' character then the
        socket will be bound using the REUSEADDR option.
@@ -207,7 +265,7 @@ class H323TransportAddress : public PString
 
     /**Create a listener compatible for this address type.
        This is similar to CreateListener() but does not use the TSAP specified
-       in the H323Transport. For example an address of "ip$10.0.0.1:1720"
+       in the H323Transport. For example an address of "#ip$10.0.0.1:1720#"
        would create a TCP listening socket that would be bound to the specific
        interface 10.0.0.1 but listens on a random OS allocated port number.
       */
@@ -226,7 +284,39 @@ class H323TransportAddress : public PString
 };
 
 
-PARRAY(H323TransportAddressArray, H323TransportAddress);
+PDECLARE_ARRAY(H323TransportAddressArray, H323TransportAddress)
+  public:
+    H323TransportAddressArray(
+      const H323TransportAddress & address
+    ) { AppendAddress(address); }
+    H323TransportAddressArray(
+      const H225_ArrayOf_TransportAddress & addresses
+    );
+    H323TransportAddressArray(
+      const PStringArray & array
+    ) { AppendStringCollection(array); }
+    H323TransportAddressArray(
+      const PStringList & list
+    ) { AppendStringCollection(list); }
+    H323TransportAddressArray(
+      const PSortedStringList & list
+    ) { AppendStringCollection(list); }
+
+    void AppendString(
+      const char * address
+    );
+    void AppendString(
+      const PString & address
+    );
+    void AppendAddress(
+      const H323TransportAddress & address
+    );
+
+  protected:
+    void AppendStringCollection(
+      const PCollection & coll
+    );
+};
 
 
 /**This class describes a "listener" on a transport protocol.
@@ -282,18 +372,11 @@ class H323Listener : public PThread
       */
     virtual H323TransportAddress GetTransportAddress() const = 0;
 
-    /**Set up a transport address PDU for RAS channel.
-      */
-    virtual void SetUpTransportPDU(
-      H225_ArrayOf_TransportAddress & pdu,  /// List of transport addresses listening on
-      const H323TransportAddress & first    /// Transport address to take precedence
-    ) = 0;
-
     /**Set up a transport address PDU for bidirectional logical channels.
       */
     virtual BOOL SetUpTransportPDU(
       H245_TransportAddress & pdu,         /// Transport addresses listening on
-      const H323TransportAddress & first  /// Transport address to take precedence
+      const H323Transport & associatedTransport /// Associated transport for precendence and translation
     ) = 0;
   //@}
 
@@ -305,6 +388,30 @@ class H323Listener : public PThread
 PLIST(H323ListenerList, H323Listener);
 
 
+/** Return a list of transport addresses corresponding to a listener list
+  */
+H323TransportAddressArray H323GetInterfaceAddresses(
+  const H323ListenerList & listeners, /// List of listeners
+  BOOL excludeLocalHost = TRUE,       /// Flag to exclude 127.0.0.1
+  H323Transport * associatedTransport = NULL
+                          /// Associated transport for precedence and translation
+);
+
+H323TransportAddressArray H323GetInterfaceAddresses(
+  const H323TransportAddress & addr,  /// Possible INADDR_ANY address
+  BOOL excludeLocalHost = TRUE,       /// Flag to exclude 127.0.0.1
+  H323Transport * associatedTransport = NULL
+                          /// Associated transport for precedence and translation
+);
+
+/**Set the PDU field for the list of transport addresses
+  */
+void H323SetTransportAddresses(
+  const H323Transport & associatedTransport,   /// Transport for NAT address translation
+  const H323TransportAddressArray & addresses, /// Addresses to set
+  H225_ArrayOf_TransportAddress & pdu          /// List of PDU transport addresses
+);
+
 
 /**This class describes a I/O transport protocol..
    A "transport" is an object that listens for incoming connections on the
@@ -332,11 +439,11 @@ class H323Transport : public PIndirectChannel
 
   /**@name Operations */
   //@{
-    /**Get the transport dependent name of the local endpoint.
+    /**Get the transport address of the local endpoint.
       */
     virtual H323TransportAddress GetLocalAddress() const = 0;
 
-    /**Get the transport dependent name of the remote endpoint.
+    /**Get the transport address of the remote endpoint.
       */
     virtual H323TransportAddress GetRemoteAddress() const = 0;
 
@@ -390,17 +497,34 @@ class H323Transport : public PIndirectChannel
       unsigned tsap
     ) const;
 
+    /// Promiscious modes for transport
+    enum PromisciousModes {
+      AcceptFromRemoteOnly,
+      AcceptFromAnyAutoSet,
+      AcceptFromAny,
+      NumPromisciousModes
+    };
+
     /**Set read to promiscuous mode.
        Normally only reads from the specifed remote address are accepted. This
-       flag allows the remote address to be automatically set to whatever the
-       sender of the last received message was.
+       flag allows packets to be accepted from any remote, provided the
+       underlying protocol can do so. For example TCP will do nothing.
+
+       The Read() call may optionally set the remote address automatically to
+       whatever the sender host of the last received message was.
 
        Default behaviour does nothing.
       */
     virtual void SetPromiscuous(
-      BOOL promiscuous
+      PromisciousModes promiscuous
     );
 
+    /**Get the transport address of the last received PDU.
+
+       Default behaviour returns GetRemoteAddress().
+      */
+    virtual H323TransportAddress GetLastReceivedAddress() const;
+
     /**Read a protocol data unit from the transport.
        This will read using the transports mechanism for PDU boundaries, for
        example UDP is a single Read() call, while for TCP there is a TPKT
@@ -470,6 +594,10 @@ class H323Transport : public PIndirectChannel
 
   /**@name Member variable access */
   //@{
+    /**Get the associated endpoint to this transport.
+      */
+    H323EndPoint & GetEndPoint() const { return endpoint; }
+
     /**Attach a thread to the transport.
       */
     void AttachThread(
@@ -554,18 +682,13 @@ class H323ListenerTCP : public H323Listener
   PCLASSINFO(H323ListenerTCP, H323Listener);
 
   public:
-    enum {
-      DefaultSignalPort = 1720,
-      DefaultT120Port = 1503
-    };
-
     /**Create a new listener for the TCP/IP protocol.
      */
     H323ListenerTCP(
-      H323EndPoint & endpoint,        /// Endpoint instance for channel
-      PIPSocket::Address binding = INADDR_ANY, /// Local interface to listen on
-      WORD port = DefaultSignalPort,  /// TCP port to listen for connections
-      BOOL exclusive = FALSE          /// Fail if listener port in use
+      H323EndPoint & endpoint,    /// Endpoint instance for channel
+      PIPSocket::Address binding, /// Local interface to listen on
+      WORD port,                  /// TCP port to listen for connections
+      BOOL exclusive = FALSE      /// Fail if listener port in use
     );
 
     /** Destroy the listener thread.
@@ -591,18 +714,11 @@ class H323ListenerTCP : public H323Listener
       */
     virtual H323TransportAddress GetTransportAddress() const;
 
-    /**Set up a transport address PDU for RAS channel.
-      */
-    virtual void SetUpTransportPDU(
-      H225_ArrayOf_TransportAddress & pdu, /// List of transport addresses listening on
-      const H323TransportAddress & first   /// Transport address to take precedence
-    );
-
     /**Set up a transport address PDU for bidirectional logical channels.
       */
     virtual BOOL SetUpTransportPDU(
       H245_TransportAddress & pdu,        /// Transport addresses listening on
-      const H323TransportAddress & first  /// Transport address to take precedence
+      const H323Transport & associatedTransport /// Associated transport for precendence and translation
     );
 
     WORD GetListenerPort() const { return listener.GetPort(); }
@@ -728,18 +844,13 @@ class H323TransportUDP : public H323TransportIP
   PCLASSINFO(H323TransportUDP, H323TransportIP);
 
   public:
-    enum {
-      MulticastRasPort = 1718,
-      DefaultRasPort = 1719
-    };
-
     /**Create a new transport channel.
      */
     H323TransportUDP(
-      H323EndPoint & endpoint,                 /// H323 End Point object
-      PIPSocket::Address binding = INADDR_ANY, /// Local interface to listen on
-      WORD port = 0,                           /// Local port to listen on
-      BOOL promiscuous = FALSE                 /// Initial state of flag
+      H323EndPoint & endpoint,                  /// H323 End Point object
+      PIPSocket::Address binding = INADDR_ANY,  /// Local interface to listen on
+      WORD localPort = 0,                       /// Local port to listen on
+      WORD remotePort = 0                       /// Remote port to connect on
     );
     ~H323TransportUDP();
 
@@ -758,15 +869,25 @@ class H323TransportUDP : public H323TransportIP
 
     /**Set read to promiscuous mode.
        Normally only reads from the specifed remote address are accepted. This
-       flag allows the remote address to be automatically set to whatever the
-       sender of the last received message was.
+       flag allows packets to be accepted from any remote, provided the
+       underlying protocol can do so.
 
-       Default behaviour does nothing.
+       The Read() call may optionally set the remote address automatically to
+       whatever the sender host of the last received message was.
+
+       Default behaviour sets the internal flag, so that Read() operates as
+       described.
       */
     virtual void SetPromiscuous(
-      BOOL promiscuous
+      PromisciousModes promiscuous
     );
 
+    /**Get the transport address of the last received PDU.
+
+       Default behaviour returns the lastReceivedAddress member variable.
+      */
+    virtual H323TransportAddress GetLastReceivedAddress() const;
+
     /**Read a protocol data unit from the transport.
        This will read using the transports mechanism for PDU boundaries, for
        example UDP is a single Read() call, while for TCP there is a TPKT
@@ -797,7 +918,8 @@ class H323TransportUDP : public H323TransportIP
     );
 
   protected:
-    BOOL promiscuousReads;
+    PromisciousModes     promiscuousReads;
+    H323TransportAddress lastReceivedAddress;
 };
 
 
