@@ -22,6 +22,25 @@
  * Contributor(s): ______________________________________.
  *
  * $Log: main.cxx,v $
+ * Revision 1.26  2002/11/13 10:13:16  rogerh
+ * Add Speex codec.
+ *
+ * Revision 1.25  2002/11/10 08:10:43  robertj
+ * Moved constants for "well known" ports to better place (OPAL change).
+ *
+ * Revision 1.24  2002/11/10 04:26:55  robertj
+ * Fixed setting of jitter paramters
+ *
+ * Revision 1.23  2002/11/04 10:27:48  rogerh
+ * Only print duration when a connection was made. (copied from ohphone)
+ *
+ * Revision 1.22  2002/10/31 00:52:18  robertj
+ * Enhanced jitter buffer system so operates dynamically between minimum and
+ *   maximum values. Altered API to assure app writers note the change!
+ *
+ * Revision 1.21  2002/07/17 05:52:21  robertj
+ * Fixed using -i parameter for interface when discovering gk, thanks Steve Frare
+ *
  * Revision 1.20  2002/02/11 06:20:07  robertj
  * Moved version.h to root directory so have one for the library and not just
  *   the sample application. SimpH323 uses the library version so they remain
@@ -178,7 +197,7 @@ void SimpleH323Process::Main()
             "  -u --user name          : Set local alias name(s) (defaults to login name).\n"
             "  -p --password pwd       : Set the H.235 password to use for calls.\n"
             "  -b --bandwidth bps      : Limit bandwidth usage to bps bits/second.\n"
-            "  -j --jitter delay       : Set jitter buffer to delay milliseconds.\n"
+            "  -j --jitter [min-]max   : Set minimum (optional) and maximum jitter buffer (in milliseconds).\n"
             "  -D --disable codec      : Disable the specified codec (may be used multiple times)\n"
             "  -P --prefer codec       : Prefer the specified codec (may be used multiple times)\n"
             "  -i --interface ipnum    : Select interface to bind to.\n"
@@ -283,11 +302,21 @@ BOOL SimpleH323EndPoint::Initialise(PArgList & args)
   }
 
   if (args.HasOption('j')) {
-    unsigned jitter = args.GetOptionString('j').AsUnsigned();
-    if (jitter >= 20 && jitter <= 10000)
-      SetMaxAudioDelayJitter(jitter);
+    unsigned minJitter;
+    unsigned maxJitter;
+    PStringArray delays = args.GetOptionString('j').Tokenise(",-");
+    if (delays.GetSize() < 2) {
+      maxJitter = delays[0].AsUnsigned();
+      minJitter = PMIN(GetMinAudioJitterDelay(), maxJitter);
+    }
     else {
-      cerr << "Jitter should be between 20 milliseconds and 10 seconds." << endl;
+      minJitter = delays[0].AsUnsigned();
+      maxJitter = delays[1].AsUnsigned();
+    }
+    if (minJitter >= 20 && minJitter <= maxJitter && maxJitter <= 1000)
+      SetAudioJitterDelay(minJitter, maxJitter);
+    else {
+      cerr << "Jitter should be between 20 and 1000 milliseconds." << endl;
       return FALSE;
     }
   }
@@ -305,6 +334,7 @@ BOOL SimpleH323EndPoint::Initialise(PArgList & args)
   // Set the default codecs available on sound cards.
   AddAllCapabilities(0, 0, "GSM*{sw}");
   AddAllCapabilities(0, 0, "G.711*{sw}");
+  AddAllCapabilities(0, 0, "Speex*{sw}");
   AddAllCapabilities(0, 0, "LPC*{sw}");
   AddAllUserInputCapabilities(0, 1);
 
@@ -316,24 +346,18 @@ BOOL SimpleH323EndPoint::Initialise(PArgList & args)
        << "Auto answer is " << autoAnswer << "\n"
        << "FastConnect is " << (IsFastStartDisabled() ? "Dis" : "En") << "abled\n"
        << "H245Tunnelling is " << (IsH245TunnelingDisabled() ? "Dis" : "En") << "abled\n"
-       << "Jitter buffer: "  << GetMaxAudioDelayJitter() << " ms\n"
+       << "Jitter buffer: "  << GetMinAudioJitterDelay() << '-' << GetMaxAudioJitterDelay() << " ms\n"
        << "Sound output device: \"" << GetSoundChannelPlayDevice() << "\"\n"
           "Sound  input device: \"" << GetSoundChannelRecordDevice() << "\"\n"
        <<  "Codecs (in preference order):\n" << setprecision(2) << GetCapabilities() << endl;
 
 
   // Start the listener thread for incoming calls.
-  H323ListenerTCP * listener;
-  if (args.GetOptionString('i').IsEmpty())
-    listener  = new H323ListenerTCP(*this);
-  else {
-    PIPSocket::Address interfaceAddress(args.GetOptionString('i'));
-    listener  = new H323ListenerTCP(*this, interfaceAddress);
-  }
-  if (!StartListener(listener)) {
-    cerr <<  "Could not open H.323 listener port on "
-         << listener->GetListenerPort() << endl;
-    delete listener;
+  H323TransportAddress iface = args.GetOptionString('i');
+  if (iface.IsEmpty())
+    iface = "*";
+  if (!StartListener(iface)) {
+    cerr <<  "Could not open H.323 listener port on \"" << iface << '"' << endl;
     return FALSE;
   }
 
@@ -345,8 +369,7 @@ BOOL SimpleH323EndPoint::Initialise(PArgList & args)
   }
 
   // Establish link with gatekeeper if required.
-  if (args.HasOption('g')) {
-    PString gkName = args.GetOptionString('g');
+  if (args.HasOption('g') || !args.HasOption('n')) {
     H323TransportUDP * rasChannel;
     if (args.GetOptionString('i').IsEmpty())
       rasChannel  = new H323TransportUDP(*this);
@@ -354,21 +377,25 @@ BOOL SimpleH323EndPoint::Initialise(PArgList & args)
       PIPSocket::Address interfaceAddress(args.GetOptionString('i'));
       rasChannel  = new H323TransportUDP(*this, interfaceAddress);
     }
-    if (SetGatekeeper(gkName, rasChannel))
-      cout << "Gatekeeper set: " << *gatekeeper << endl;
-    else {
-      cerr << "Error registering with gatekeeper at \"" << gkName << '"' << endl;
-      return FALSE;
-    }
-  }
-  else if (!args.HasOption('n') || args.HasOption('r')) {
-    cout << "Searching for gatekeeper..." << flush;
-    if (DiscoverGatekeeper(new H323TransportUDP(*this)))
-      cout << "\nGatekeeper found: " << *gatekeeper << endl;
-    else {
-      cerr << "\nNo gatekeeper found." << endl;
-      if (args.HasOption("require-gatekeeper")) 
+
+    if (args.HasOption('g')) {
+      PString gkName = args.GetOptionString('g');
+      if (SetGatekeeper(gkName, rasChannel))
+        cout << "Gatekeeper set: " << *gatekeeper << endl;
+      else {
+        cerr << "Error registering with gatekeeper at \"" << gkName << '"' << endl;
         return FALSE;
+      }
+    }
+    else {
+      cout << "Searching for gatekeeper..." << flush;
+      if (DiscoverGatekeeper(rasChannel))
+        cout << "\nGatekeeper found: " << *gatekeeper << endl;
+      else {
+        cerr << "\nNo gatekeeper found." << endl;
+        if (args.HasOption('r')) 
+          return FALSE;
+      }
     }
   }
 
@@ -524,10 +551,13 @@ void SimpleH323EndPoint::OnConnectionCleared(H323Connection & connection,
     default :
       cout << "Call with " << remoteName << " completed";
   }
-  cout << ", duration "
-       << setprecision(0) << setw(5)
-       << (PTime() - connection.GetConnectionStartTime())
-       << endl;
+  PTime connectTime = connection.GetConnectionStartTime();
+  if (connectTime.GetTimeInSeconds() != 0)
+    cout << ", duration "
+         << setprecision(0) << setw(5)
+         << (PTime() - connectTime);
+
+  cout << endl;
 }
 
 
