@@ -27,6 +27,12 @@
  * Contributor(s): ______________________________________.
  *
  * $Log: codecs.cxx,v $
+ * Revision 1.83  2004/02/04 10:29:27  rjongbloed
+ * Fixed G.726 by allowing for more bits per pixels sizes in streamed codec, thanks Kevin Bouchard
+ *
+ * Revision 1.82  2003/11/12 11:14:51  csoutheren
+ * Added H323FramedAudioCodec::DecodeSilenceFrame thanks to Henry Harrison of AliceStreet
+ *
  * Revision 1.81  2003/07/16 10:43:13  csoutheren
  * Added SwapChannel function to H323Codec to allow media hold channels
  * to work better. Thanks to Federico Pinna
@@ -950,8 +956,9 @@ BOOL H323FramedAudioCodec::Write(const BYTE * buffer,
     }
   }
 
+  // was memset(sampleBuffer.GetPointer(samplesPerFrame), 0, bytesOfPCM);
   if (length == 0)
-    memset(sampleBuffer.GetPointer(samplesPerFrame), 0, bytesOfPCM);
+    DecodeSilenceFrame(sampleBuffer.GetPointer(bytesOfPCM), bytesOfPCM);
 
   // Write as 16bit PCM to sound channel
   return WriteRaw(sampleBuffer.GetPointer(), bytesOfPCM);
@@ -993,12 +1000,59 @@ H323StreamedAudioCodec::H323StreamedAudioCodec(const char * fmt,
 BOOL H323StreamedAudioCodec::EncodeFrame(BYTE * buffer, unsigned &)
 {
   PINDEX i;
-
+  unsigned short position = 0;
+  BYTE encoded;
   switch (bitsPerSample) {
     case 8 :
       for (i = 0; i < (PINDEX)samplesPerFrame; i++)
         *buffer++ = (BYTE)Encode(sampleBuffer[i]);
       break;
+    case 5 : // g.726-40 payload encoding....
+      for (i = 0; i < (PINDEX)samplesPerFrame;i++)
+      {
+        // based on a 40 bits encoding, we have 8 words of 5 bits each
+        encoded = (BYTE)Encode(sampleBuffer[i]);
+        switch(position)
+        {
+          case 0: // 0 bits overflow
+            *buffer = encoded;
+            position++;
+            break;
+          case 1: // 2 bits overflow
+            *buffer++ |= (encoded << 5);
+            *buffer = (BYTE)(encoded >> 3);
+            position++;
+            break;
+          case 2: 
+            *buffer |= (encoded << 2);
+            position++;
+            break;
+          case 3: // one bit left for word 4
+            *buffer++ |= (encoded << 7);
+            *buffer = (BYTE)(encoded >> 1);
+            position++;
+            break;
+          case 4:
+            *buffer++ |= (encoded << 4);
+            *buffer = (BYTE)(encoded >> 4);
+            position++;
+            break;
+          case 5:
+            *buffer |= (encoded << 1);
+            position++;
+            break;
+          case 6: //two bits left for the new encoded word
+            *buffer++ |= (encoded << 6);
+            *buffer =  (BYTE)(encoded >> 2);
+            position++;
+            break;
+          case 7: // now five bits left for the last word
+            *buffer++ |= (encoded << 3);
+            position = 0;
+            break;
+        }
+      }
+      break;
 
     case 4 :
       for (i = 0; i < (PINDEX)samplesPerFrame; i++) {
@@ -1009,11 +1063,80 @@ BOOL H323StreamedAudioCodec::EncodeFrame(BYTE * buffer, unsigned &)
       }
       break;
 
+    case 3 :
+      for (i = 0;i < (PINDEX)samplesPerFrame;i++)
+      {
+        encoded = (BYTE)Encode(sampleBuffer[i]);
+        switch(position)
+        {
+          case 0: // 0 bits overflow
+            *buffer = encoded;
+            position++;
+            break;
+          case 1: // 2 bits overflow
+            *buffer |= (encoded << 3);
+            position++;
+            break;
+          case 2: 
+            *buffer++ |= (encoded << 6);
+            *buffer = (BYTE)(encoded >> 2);
+            position++;
+            break;
+          case 3: // one bit left for word 4
+            *buffer |= (encoded << 1);
+            position++;
+            break;
+          case 4:
+            *buffer |= (encoded << 4);
+            position++;
+            break;
+          case 5:
+            *buffer++ |= (encoded << 7);
+            *buffer = (BYTE)(encoded >> 1);
+            position++;
+            break;
+          case 6: //two bits left for the new encoded word
+            *buffer |= (encoded << 2);
+            position++;
+            break;
+          case 7: // now five bits left for the last word
+            *buffer++ |= (encoded << 5);
+            position = 0;
+            break;
+        }
+      }
+      break;
+
+    case 2:
+      for (i = 0; i < (PINDEX)samplesPerFrame; i++) 
+      {
+        switch(position)
+        {
+          case 0:
+            *buffer = (BYTE)Encode(sampleBuffer[i]);
+            position++;
+            break;
+          case 1:
+            *buffer |= (BYTE)(Encode(sampleBuffer[i]) << 2);
+            position++;
+            break;
+          case 2:
+            *buffer |= (BYTE)(Encode(sampleBuffer[i]) << 4);
+            position++;
+            break;
+          case 3:
+            *buffer++ |= (BYTE)(Encode(sampleBuffer[i]) << 6);
+            position = 0;
+            break;
+        }
+      }
+      break;
+
     default :
       PAssertAlways("Unsupported bit size");
       return FALSE;
   }
-
+  
   return TRUE;
 }
 
@@ -1021,14 +1144,58 @@ BOOL H323StreamedAudioCodec::EncodeFrame(BYTE * buffer, unsigned &)
 BOOL H323StreamedAudioCodec::DecodeFrame(const BYTE * buffer, unsigned length, unsigned & written)
 {
   unsigned i;
+  
   short * out = sampleBuffer.GetPointer(samplesPerFrame);
-
+  unsigned short position = 0;
+  unsigned remaining = 0;
+  
   switch (bitsPerSample) {
     case 8 :
       for (i = 0; i < length; i++)
         *out++ = Decode(*buffer++);
       break;
 
+    // those case are for ADPCM G.726
+    case 5 :
+      for (i = 0; i < length; i++) {
+        switch(position)
+        {
+          case 0:
+            *out++ = Decode(*buffer & 31);
+            remaining = *buffer >> 5; // get the three remaining bytes for the next word
+            buffer++;
+            position++;
+            break;
+          case 1: // we can decode more than one word in second buffer
+            *out++ = Decode (((*buffer&3) << 3) | remaining);
+            *out++ = Decode( (*buffer >> 2) & 31);
+            remaining = *buffer >> 7;
+            buffer++;
+            position++;
+            break;
+          case 2:
+            *out++ = Decode( remaining | ((*buffer&15) << 1));
+            remaining = *buffer >> 4;
+            buffer++;
+            position++;
+            break;
+          case 3:
+            *out++ = Decode( remaining | ((*buffer&1) << 4));
+            *out++ = Decode( (*buffer >> 1) & 31);
+            remaining = *buffer >> 6;
+            buffer++;
+            position++;
+            break;
+          case 4 :
+            *out++ = Decode( remaining | ((*buffer&7) << 2));
+            *out++ = Decode(*buffer >> 3);
+            buffer++;
+            position = 0;
+            break;
+        }
+      }
+      break;
+
     case 4 :
       for (i = 0; i < length; i++) {
         *out++ = Decode(*buffer & 15);
@@ -1037,13 +1204,55 @@ BOOL H323StreamedAudioCodec::DecodeFrame(const BYTE * buffer, unsigned length, u
       }
       break;
 
+    case 3:
+      for (i = 0; i < length; i++) {
+        switch(position)
+        {
+        case 0:
+          *out++ = Decode(*buffer & 7);
+          *out++ = Decode((*buffer>>3)&7);
+          remaining = *buffer >> 6;
+          buffer++;
+          position++;
+          break;
+        case 1:
+          *out++ = Decode(remaining | ((*buffer&1) << 2));
+          *out++ = Decode((*buffer >> 1) & 7);
+          *out++ = Decode((*buffer >> 4)&7);
+          remaining = *buffer >> 7;
+          buffer++;
+          position++;
+          break;
+        case 2:
+          *out++ = Decode(remaining | ((*buffer&3) << 1));
+          *out++ = Decode((*buffer >> 2) & 7);
+          *out++ = Decode((*buffer >> 5) & 7);
+          buffer++;
+          position = 0;
+          break;
+        }
+      }
+      break;
+
+    case 2:
+      for (i = 0; i < length; i++) 
+      {
+        *out++ = Decode(*buffer & 3);
+        *out++ = Decode((*buffer >> 2) & 3);
+        *out++ = Decode((*buffer >> 4) & 3);
+        *out++ = Decode((*buffer >> 6) & 3);
+        buffer++;
+      
+      }
+      break;
+
     default :
       PAssertAlways("Unsupported bit size");
       return FALSE;
   }
-
+  
   written = length;
-
+  
   return TRUE;
 }
 
