@@ -24,6 +24,9 @@
  * Contributor(s): ______________________________________.
  *
  * $Log: g726codec.cxx,v $
+ * Revision 1.9  2004/03/08 01:20:09  csoutheren
+ * Fixed problems that allowed samplesPerFrames to be set to zero, causing crash
+ *
  * Revision 1.8  2004/02/17 10:47:55  csoutheren
  * Fixed compile problem on Windows
  *
@@ -122,22 +125,25 @@ H323_REGISTER_CAPABILITY_FUNCTION(H323_G726_16_Capability, OPAL_G726_16"{sw}", e
 
 struct G726_NonStandardInfo {
   char name[10]; /// G.726-xxk
-  BYTE count;
+  BYTE maxCount;
+  BYTE desiredCount;
 };
 
 
 static G726_NonStandardInfo const G726_NonStandard[H323_G726_Capability::NumSpeeds] = {
-  { OPAL_G726_40 },
-  { OPAL_G726_32 },
-  { OPAL_G726_24 },
-  { OPAL_G726_16 }
+  { OPAL_G726_40, 240, 10 },
+  { OPAL_G726_32, 240, 10 },
+  { OPAL_G726_24, 240, 10 },
+  { OPAL_G726_16, 240, 10 }
 };
 
 
 /////////////////////////////////////////////////////////////////////////////
 
 H323_G726_Capability::H323_G726_Capability(H323EndPoint & endpoint, Speeds s)
-    : H323NonStandardAudioCapability(240, 10, endpoint,
+    : H323NonStandardAudioCapability(G726_NonStandard[s].maxCount,
+                                     G726_NonStandard[s].desiredCount, 
+                                     endpoint,
                                      (const BYTE *)&(G726_NonStandard[s].name),
                                      strlen(G726_NonStandard[s].name),
                                      0, strlen(G726_NonStandard[s].name))
@@ -157,12 +163,17 @@ PString H323_G726_Capability::GetFormatName() const
   return PString(G726_NonStandard[speed].name) + "{sw}";
 }
 
+void H323_G726_Capability::SetTxFramesInPacket(unsigned frames)
+{
+  if (frames > G726_NonStandard[speed].maxCount)
+    txFramesInPacket = G726_NonStandard[speed].maxCount;
+  else
+    txFramesInPacket = frames;
+}
 
 BOOL H323_G726_Capability::OnSendingPDU(H245_AudioCapability & pdu,
                                         unsigned packetSize) const
 {
-  G726_NonStandardInfo * info = (G726_NonStandardInfo *)(const BYTE *)nonStandardData;
-  info->count = (BYTE)packetSize;
   return H323NonStandardAudioCapability::OnSendingPDU(pdu, packetSize);
 }
 
@@ -173,8 +184,7 @@ BOOL H323_G726_Capability::OnReceivedPDU(const H245_AudioCapability & pdu,
   if (!H323NonStandardAudioCapability::OnReceivedPDU(pdu, packetSize))
     return FALSE;
 
-  const G726_NonStandardInfo * info = (const G726_NonStandardInfo *)(const BYTE *)nonStandardData;
-  packetSize = info->count;
+  packetSize = G726_NonStandard[speed].maxCount;
   return TRUE;
 }
 
@@ -182,7 +192,6 @@ BOOL H323_G726_Capability::OnReceivedPDU(const H245_AudioCapability & pdu,
 H323Codec * H323_G726_Capability::CreateCodec(H323Codec::Direction direction) const
 {
   unsigned packetSize = 8*(direction == H323Codec::Encoder ? txFramesInPacket : rxFramesInPacket);
-
   return new H323_G726_Codec(speed, direction, packetSize);
 }
 
@@ -248,8 +257,5 @@ short H323_G726_Codec::Decode(int sample) const
   return 0;
 }
 
-
 /////////////////////////////////////////////////////////////////////////////
 
-
-
