@@ -27,6 +27,15 @@
  * Contributor(s): ______________________________________.
  *
  * $Log: gkclient.cxx,v $
+ * Revision 1.148  2004/01/17 18:20:15  csoutheren
+ * No longer force re-register on completion of LRQ
+ *
+ * Revision 1.147  2003/12/29 04:59:25  csoutheren
+ * Added callbacks on H323EndPoint when gatekeeper discovery succeeds or fails
+ *
+ * Revision 1.146  2003/12/28 00:06:34  csoutheren
+ * Added callbacks on H323EndPoint when gatekeeper registration succeeds or fails
+ *
  * Revision 1.145  2003/05/01 05:04:00  robertj
  * Fixed inclusion of 127.0.0.1 into listener lists when no needed.
  *
@@ -740,6 +749,8 @@ BOOL H323Gatekeeper::OnReceiveGatekeeperConfirm(const H225_GatekeeperConfirm & g
   if (gcf.HasOptionalField(H225_GatekeeperConfirm::e_alternateGatekeeper))
     SetAlternates(gcf.m_alternateGatekeeper, FALSE);
 
+  endpoint.OnGatekeeperConfirm();
+
   discoveryComplete = TRUE;
   return TRUE;
 }
@@ -760,6 +771,8 @@ BOOL H323Gatekeeper::OnReceiveGatekeeperReject(const H225_GatekeeperReject & grj
     }
   }
 
+  endpoint.OnGatekeeperReject();
+
   return TRUE;
 }
 
@@ -973,6 +986,8 @@ BOOL H323Gatekeeper::OnReceiveRegistrationConfirm(const H225_RegistrationConfirm
   if (rcf.HasOptionalField(H225_RegistrationConfirm::e_serviceControl))
     OnServiceControlSessions(rcf.m_serviceControl, NULL);
 
+  endpoint.OnRegistrationConfirm();
+
   return TRUE;
 }
 
@@ -986,6 +1001,8 @@ BOOL H323Gatekeeper::OnReceiveRegistrationReject(const H225_RegistrationReject &
     SetAlternates(rrj.m_altGKInfo.m_alternateGatekeeper,
                   rrj.m_altGKInfo.m_altGKisPermanent);
 
+  endpoint.OnRegistrationReject();
+
   return TRUE;
 }
 
@@ -1187,7 +1204,7 @@ BOOL H323Gatekeeper::LocationRequest(const PStringList & aliases,
 
   Request request(lrq.m_requestSeqNum, pdu);
   request.responseInfo = &address;
-  return MakeRequestWithReregister(request, H225_LocationRejectReason::e_notRegistered);
+  return MakeRequest(request);
 }
 
 
