@@ -1,5 +1,5 @@
 /*
- * h263codec.cxx
+ * h263codec.cxx   
  *
  * H.323 protocol handler
  *
@@ -27,6 +27,9 @@
  * Contributor(s): Guilhem Tardy (gtardy@marchnetworks.com)
  *
  * $Log: h263codec.cxx,v $
+ * Revision 1.24  2003/08/29 00:33:22  dereksmithies
+ * Add fix for printing vic messages to the log file. MANY thanks to Niklas Ogren
+ *
  * Revision 1.23  2003/08/04 00:03:41  dereksmithies
  * Reorganise test of H323_VICH263 switch
  *
@@ -164,6 +167,7 @@ VicH263Link::VicH263Link()
   vich263_set_print_fn(h263_vic_printon);
 }
 
+VicH263Link vicLink;
 
 ///////////////////////////////////////////////////////////////////////////////////////
 
@@ -447,7 +451,6 @@ H323_H263Codec::H323_H263Codec(Direction dir,
 	 << "coder created. Data rate=" << _maxBitRate
 	 << " Frame rate=" << _videoFrameRate);
 
-
   bitRateHighLimit = _maxBitRate;
   if (bitRateHighLimit == 0) {
     PTRACE(3, "H263\tData Rate is set to 1000 Kb/sec, as supplied value (0) is invalid");
@@ -510,7 +513,7 @@ H323_H263Codec::~H323_H263Codec()
 }
 
 void H323_H263Codec::InitialiseCodec()
-{
+{ 
    // no video decoder until we receive a packet
    videoDecoder = NULL;
 
