@@ -24,6 +24,13 @@
  * Contributor(s): ______________________________________.
  *
  * $Log: h323trans.cxx,v $
+ * Revision 1.23  2003/12/11 05:41:00  csoutheren
+ * Added storage of H.225 version in endpoint structure
+ * Disabled sending RIPs to endpoints that cannot handle them
+ *
+ * Revision 1.22  2003/10/27 01:55:33  csoutheren
+ * Guarded against possible NULL interface
+ *
  * Revision 1.21  2003/04/30 07:51:08  robertj
  * Redesigned the alternate credentials in ARQ system as old implementation
  *   was fraught with concurrency issues, most importantly it can cause false
@@ -281,7 +288,7 @@ BOOL H323Transactor::SetTransport(const H323TransportAddress & iface)
     delete transport;
   }
 
-  transport = new H323TransportUDP(endpoint, addr, port, defaultRemotePort);
+    transport = new H323TransportUDP(endpoint, addr, port, defaultRemotePort);
   transport->SetPromiscuous(H323Transport::AcceptFromAny);
   return StartChannel();;
 }
@@ -289,9 +296,12 @@ BOOL H323Transactor::SetTransport(const H323TransportAddress & iface)
 H323TransportAddressArray H323Transactor::GetInterfaceAddresses(BOOL excludeLocalHost,
                                                                 H323Transport * associatedTransport)
 {
-  return H323GetInterfaceAddresses(transport->GetLocalAddress(),
-                                   excludeLocalHost,
-                                   associatedTransport);
+  if (transport == NULL)
+    return H323TransportAddressArray();
+  else
+    return H323GetInterfaceAddresses(transport->GetLocalAddress(),
+                                     excludeLocalHost,
+                                     associatedTransport);
 }
 
 BOOL H323Transactor::StartChannel()
@@ -782,6 +792,7 @@ H323Transaction::H323Transaction(H323Transactor & trans,
   authenticatorResult = H235Authenticator::e_Disabled;
   fastResponseRequired = TRUE;
   isBehindNAT = FALSE;
+  canSendRIP  = FALSE;
 }
 
 
