@@ -24,6 +24,14 @@
  * Contributor(s): ______________________________________.
  *
  * $Log: h450pdu.cxx,v $
+ * Revision 1.22  2004/02/17 09:53:54  csoutheren
+ * Removed incorrect assignment of m_silentMonitoringPermitted to FALSE
+ * See SourceForge bug 832371
+ * Thanks to Vyacheslav E. Andrejev
+ *
+ * Revision 1.21  2004/02/07 03:35:56  rjongbloed
+ * Fixed missing unlocks in some H.240 operations, in particular consultation transfer never being cleaned up. Thanks Sbastien Annedouche
+ *
  * Revision 1.20  2003/07/15 11:22:07  csoutheren
  * Improved handling of alias addresses thanks to Federico Pinna
  *
@@ -1206,13 +1214,14 @@ void H4502Handler::OnReceivedIdentifyReturnResult(X880_ReturnResult &returnResul
     // callTransferAbandon invoke APDU on the secondary call at a later stage if we 
     // get back a callTransferInitiateReturnError
     H323Connection* primaryConnection = endpoint.FindConnectionWithLock(CallToken);
-
     if (primaryConnection != NULL) {
       primaryConnection->SetAssociatedCallToken(connection.GetCallToken());
 
       // Send a callTransferInitiate invoke APDU in a FACILITY message
       // to the transferred endpoint on the primary call
       endpoint.TransferCall(primaryConnection->GetCallToken(), remoteParty, callIdentity);
+
+      primaryConnection->Unlock();
     }
   }
 }
@@ -1979,7 +1988,6 @@ void H45011Handler::OnReceivedCallIntrusionGetCIPL(int /*linkedId*/,
 
   ciCIPLRes.m_ciProtectionLevel = endpoint.GetCallIntrusionProtectionLevel();
   ciCIPLRes.IncludeOptionalField(H45011_CIGetCIPLRes::e_silentMonitoringPermitted);
-  ciCIPLRes.m_silentMonitoringPermitted = FALSE;
 
   PPER_Stream resultStream;
   ciCIPLRes.Encode(resultStream);
@@ -2229,6 +2237,7 @@ void H45011Handler::OnReceivedCIGetCIPLResult(X880_ReturnResult & returnResult)
       // Clear Call with intruding (A)
       H323Connection* conn = endpoint.FindConnectionWithLock(intrudingCallToken);
       conn->SetIntrusionNotAuthorized();
+      conn->Unlock();
       endpoint.ClearCall (intrudingCallToken);
     }
   }
