@@ -27,6 +27,28 @@
  * Contributor(s): ______________________________________.
  *
  * $Log: ixjwin32.cxx,v $
+ * Revision 1.114  2003/04/29 08:32:59  robertj
+ * Added new wink functions for Windows IxJ lid.
+ *
+ * Revision 1.113  2002/11/05 04:33:21  robertj
+ * Changed IsLineDisconnected() to work with POTSLine
+ *
+ * Revision 1.112  2002/11/05 04:27:58  robertj
+ * Imported RingLine() by array from OPAL.
+ *
+ * Revision 1.111  2002/10/30 05:54:17  craigs
+ * Fixed compatibilty problems with G.723.1 6k3 and 5k3
+ *
+ * Revision 1.110  2002/06/25 08:30:12  robertj
+ * Changes to differentiate between stright G.723.1 and G.723.1 Annex A using
+ *   the OLC dataType silenceSuppression field so does not send SID frames
+ *   to receiver codecs that do not understand them.
+ *
+ * Revision 1.109  2002/05/09 06:26:34  robertj
+ * Added fuction to get the current audio enable state for line in device.
+ * Changed IxJ EnableAudio() semantics so is exclusive, no direct switching
+ *   from PSTN to POTS and vice versa without disabling the old one first.
+ *
  * Revision 1.108  2002/03/21 02:37:33  robertj
  * Fixed G.723.1 5.3k mode so receiver (playback) still accepts 6.3k data.
  *
@@ -428,6 +450,8 @@ OpalIxJDevice::OpalIxJDevice()
   readCodecType = writeCodecType = P_MAX_INDEX;
   currentHookState = lastHookState = FALSE;
   inRawMode = FALSE;
+  enabledAudioLine = UINT_MAX;
+  exclusiveAudioMode = TRUE;
   lastDTMFDigit = 0;
   hReadEvent = hWriteEvent = NULL;
 }
@@ -724,16 +748,57 @@ BOOL OpalIxJDevice::RingLine(unsigned line, DWORD cadence)
   if (line != POTSLine)
     return FALSE;
 
+  if (cadence == TRUE) {
+    switch (countryCode) {
+      case Australia :
+        static unsigned AusRing[] = { 200, 400, 200, 2000 };
+        return RingLine(line, PARRAYSIZE(AusRing), AusRing);
+    }
+
+    // United States ring pattern
+    cadence = 0x00f;
+  }
+
   IoControl(IOCTL_DevCtrl_SetPotsToSlic, 1);
   return IoControl(IOCTL_DevCtrl_SetRingPattern, cadence);
 }
 
 
+BOOL OpalIxJDevice::RingLine(unsigned line, PINDEX nCadence, unsigned * pattern)
+{
+  if (line >= GetLineCount())
+    return FALSE;
+
+  if (line != POTSLine)
+    return FALSE;
+
+  IoControl(IOCTL_DevCtrl_SetPotsToSlic, 1);
+
+
+  DWORD dwReturn, dwSize;
+  DWORD cadenceArray[10];
+  cadenceArray[0] = (nCadence+1)/2; // Number of pairs
+  cadenceArray[1] = (nCadence&1) == 0; // If odd then repeat last entry
+  PINDEX i;
+  for (i = 2; i < nCadence; i++)
+    cadenceArray[i] = pattern[i-2];
+  for (; i < PARRAYSIZE(cadenceArray); i++)
+    cadenceArray[i] = 0;
+
+  return IoControl(IOCTL_DevCtrl_SetRingCadence,
+                   cadenceArray, sizeof(cadenceArray),
+                   &dwReturn, sizeof(dwReturn), &dwSize);
+}
+
+
 BOOL OpalIxJDevice::IsLineDisconnected(unsigned line, BOOL checkForWink)
 {
-  if ((line >= GetLineCount()) || (line != PSTNLine))
+  if (line >= GetLineCount())
     return FALSE;
 
+  if (line != PSTNLine)
+    return !IsLineOffHook(line);
+
   if (checkForWink) {
     DWORD dwResult = 0;
     if (IoControl(IOCTL_DevCtrl_GetLineCallerOnHook, 0, &dwResult) && dwResult != 0) {
@@ -805,8 +870,14 @@ static const struct {
   { OPAL_G728,          2, 0, 0, 0,  20, RECORD_MODE_TRUESPEECH, RECORD_RATE_G728,    PLAYBACK_MODE_TRUESPEECH, PLAYBACK_RATE_G728    },
   { OPAL_G729,          6, 1, 0, 0,  10, RECORD_MODE_TRUESPEECH, RECORD_RATE_G729,    PLAYBACK_MODE_TRUESPEECH, PLAYBACK_RATE_G729    },
   { OPAL_G729AB,        6, 1, 0, 1,  10, RECORD_MODE_TRUESPEECH, RECORD_RATE_G729,    PLAYBACK_MODE_TRUESPEECH, PLAYBACK_RATE_G729    },
-  { OPAL_G7231_6k3,     7, 0, 1, 1,  24, RECORD_MODE_TRUESPEECH, RECORD_RATE_G723_63, PLAYBACK_MODE_TRUESPEECH, PLAYBACK_RATE_G723_63 },
-  { OPAL_G7231_5k3,     7, 0, 1, 1,  20, RECORD_MODE_TRUESPEECH, RECORD_RATE_G723_53, PLAYBACK_MODE_TRUESPEECH, PLAYBACK_RATE_G723_63 },
+
+  // these two lines should be for the 5k3 codec, but this does not work properly in the driver so we lie
+  { OPAL_G7231_5k3,     7, 0, 1, 0,  24, RECORD_MODE_TRUESPEECH, RECORD_RATE_G723_63, PLAYBACK_MODE_TRUESPEECH, PLAYBACK_RATE_G723_63 },
+  { OPAL_G7231A_5k3,    7, 0, 1, 1,  24, RECORD_MODE_TRUESPEECH, RECORD_RATE_G723_63, PLAYBACK_MODE_TRUESPEECH, PLAYBACK_RATE_G723_63 },
+
+  { OPAL_G7231_6k3,     7, 0, 1, 0,  24, RECORD_MODE_TRUESPEECH, RECORD_RATE_G723_63, PLAYBACK_MODE_TRUESPEECH, PLAYBACK_RATE_G723_63 },
+  { OPAL_G7231A_6k3,    7, 0, 1, 1,  24, RECORD_MODE_TRUESPEECH, RECORD_RATE_G723_63, PLAYBACK_MODE_TRUESPEECH, PLAYBACK_RATE_G723_63 },
+
 };
 
 
@@ -1339,8 +1410,18 @@ BOOL OpalIxJDevice::EnableAudio(unsigned line, BOOL enable)
 
   DWORD dwSource = ANALOG_SOURCE_SPEAKERPHONE;
 
-  if (enable)
+  if (enable) {
+    if (enabledAudioLine != line) {
+      if (enabledAudioLine != UINT_MAX && exclusiveAudioMode) {
+        PTRACE(3, "xJack\tEnableAudio on port when already enabled other port.");
+        return FALSE;
+      }
+      enabledAudioLine = line;
+    }
     dwSource = line == POTSLine ? ANALOG_SOURCE_POTSPHONE : ANALOG_SOURCE_PSTNLINE;
+  }
+  else
+    enabledAudioLine = UINT_MAX;
 
   if (!IsLineJACK())
     return IoControl(IOCTL_DevCtrl_SetAnalogSource, dwSource);
@@ -1367,6 +1448,12 @@ BOOL OpalIxJDevice::EnableAudio(unsigned line, BOOL enable)
 }
 
 
+BOOL OpalIxJDevice::IsAudioEnabled(unsigned line)
+{
+  return enabledAudioLine == line;
+}
+
+
 BOOL OpalIxJDevice::InternalSetVolume(BOOL record, unsigned id, int volume, int mute)
 {
   MIXER_LINE mixer;
@@ -1473,6 +1560,20 @@ BOOL OpalIxJDevice::SetAEC(unsigned, AECLevels level)
 }
 
 
+unsigned OpalIxJDevice::GetWinkDuration(unsigned)
+{
+  DWORD level = 0;
+  IoControl(IOCTL_DevCtrl_GetLineWinkDetTime, 0, &level);
+  return (unsigned)level;
+}
+
+
+BOOL OpalIxJDevice::SetWinkDuration(unsigned, unsigned winkDuration)
+{
+  return IoControl(IOCTL_DevCtrl_SetLineWinkDetTime, winkDuration);
+}
+
+
 BOOL OpalIxJDevice::GetVAD(unsigned)
 {
   return vadEnabled;
@@ -1481,6 +1582,7 @@ BOOL OpalIxJDevice::GetVAD(unsigned)
 
 BOOL OpalIxJDevice::SetVAD(unsigned, BOOL enable)
 {
+  PTRACE(3, "xJack\tSet VAD " << (enable ? "on" : "off"));
   vadEnabled = enable;
   return IoControl(enable ? IOCTL_Record_EnableVAD : IOCTL_Record_DisableVAD);
 }
@@ -1936,6 +2038,8 @@ BOOL OpalIxJDevice::PlayTone(unsigned line, CallProgressTones tone)
       return InternalPlayTone(line, IDLE_TONE_RING, 0, 0, FALSE);
     case BusyTone :
       return InternalPlayTone(line, IDLE_TONE_BUSY, 0, 0, FALSE);
+    case ClearTone :
+      return InternalPlayTone(line, IDLE_TONE_BUSY, 0, 0, FALSE);
     default :
       return InternalPlayTone(line, IDLE_TONE_NOTONE, 0, 0, FALSE);
   }
