@@ -1,6 +1,9 @@
 /*
 
 $Log: f2clib.c,v $
+Revision 1.3  2004/02/17 09:21:45  csoutheren
+Fix for GM bug 134591
+
 Revision 1.2  2002/02/15 03:57:55  yurik
 Warnings removed during compilation, patch courtesy of Jehan Bing, jehan@bravobrava.com
 
@@ -78,5 +81,5 @@ integer i_nint(real *x)
 #endif
 {
 return (integer)( (*x)>=0 ?
-	floor(*x + .5) : -floor(.5 - *x) );
+	floor(*x + .5) : -(floor(.5 - *x)) );
 }
