@@ -27,6 +27,60 @@
  * Contributor(s): ______________________________________.
  *
  * $Log: q931.cxx,v $
+ * Revision 1.59  2003/08/24 23:10:30  rjongbloed
+ * Fixed incorrect mask in bearer caps decoding, thanks Greg Adams
+ *
+ * Revision 1.58  2003/03/18 05:54:18  robertj
+ * Added ChannelIdentifier IE support, thanks Eize Slange
+ *
+ * Revision 1.57  2003/02/12 02:23:01  robertj
+ * Fixed printing of cause codes that are impossible as opposed to merely
+ *   unlikely or unallocated.
+ *
+ * Revision 1.56  2003/02/12 00:02:23  robertj
+ * Added more Q.931 cause codes.
+ * Added ability to trace text version of cause codes and IE codes.
+ *
+ * Revision 1.55  2002/11/19 06:19:25  robertj
+ * Added extra "congested" Q.931 codes.
+ *
+ * Revision 1.54  2002/08/06 02:27:39  robertj
+ * GNU C++ v3 compatibility.
+ *
+ * Revision 1.53  2002/08/05 10:03:48  robertj
+ * Cosmetic changes to normalise the usage of pragma interface/implementation.
+ *
+ * Revision 1.52  2002/07/25 10:55:44  robertj
+ * Changes to allow more granularity in PDU dumps, hex output increasing
+ *   with increasing trace level.
+ *
+ * Revision 1.51  2002/07/05 03:13:38  robertj
+ * Added copy constructor for Q.931 so makes duplicate instead of reference to IE's.
+ *
+ * Revision 1.50  2002/07/03 10:26:05  robertj
+ * Fixed bearer capabilities for H,450.1 needing non ITU standard, thanks Florian Winterstein
+ *
+ * Revision 1.49  2002/05/22 23:12:03  robertj
+ * Enhanced the display of Release-Complete cause codes.
+ *
+ * Revision 1.48  2002/05/07 23:49:35  craigs
+ * Changed comment on length of userUserIE field thanks to Paul Long
+ *
+ * Revision 1.47  2002/05/03 05:38:19  robertj
+ * Added Q.931 Keypad IE mechanism for user indications (DTMF).
+ *
+ * Revision 1.46  2002/04/22 07:32:16  craigs
+ * Changed GetProgressIndicator to be const
+ *
+ * Revision 1.45  2002/04/19 04:49:14  robertj
+ * Fixed currect bit shift for CallState IE standard bits.
+ *
+ * Revision 1.44  2002/04/19 02:16:47  robertj
+ * Added CallState IE processing.
+ *
+ * Revision 1.43  2002/04/18 06:16:32  craigs
+ * Extra robustness in handling of strange UserUserIE lengths
+ *
  * Revision 1.42  2002/01/07 04:25:21  robertj
  * Added support for Connected-Number Information Element, thanks Hans Verbeek
  *
@@ -163,17 +217,102 @@
  *
  */
 
+#include <ptlib.h>
+
 #ifdef __GNUC__
 #pragma implementation "q931.h"
 #endif
 
-#include <ptlib.h>
-#include <ptclib/random.h>
 #include "q931.h"
 
+#include <ptclib/random.h>
+
+
 #define new PNEW
 
 
+ostream & operator<<(ostream & strm, Q931::InformationElementCodes ie)
+{
+  static POrdinalToString::Initialiser IENamesInit[] = {
+    { Q931::BearerCapabilityIE,     "Bearer-Capability"     },
+    { Q931::CauseIE,                "Cause"                 },
+    { Q931::FacilityIE,             "Facility"              },
+    { Q931::ProgressIndicatorIE,    "Progress-Indicator"    },
+    { Q931::CallStateIE,            "Call-State"            },
+    { Q931::DisplayIE,              "Display"               },
+    { Q931::SignalIE,               "Signal"                },
+    { Q931::KeypadIE,               "Keypad"                },
+    { Q931::ConnectedNumberIE,      "Connected-Number"      },
+    { Q931::CallingPartyNumberIE,   "Calling-Party-Number"  },
+    { Q931::CalledPartyNumberIE,    "Called-Party-Number"   },
+    { Q931::RedirectingNumberIE,    "Redirecting-Number"    },
+    { Q931::ChannelIdentificationIE,"Channel-Identification"},
+    { Q931::UserUserIE,             "User-User"             } 
+  };
+  static const POrdinalToString IENames(PARRAYSIZE(IENamesInit), IENamesInit);
+
+  if (IENames.Contains((PINDEX)ie))
+    strm << IENames[ie];
+  else
+    strm << "0x" << hex << (unsigned)ie << dec << " (" << (unsigned)ie << ')';
+
+  return strm;
+}
+
+
+ostream & operator<<(ostream & strm, Q931::CauseValues cause)
+{
+  static POrdinalToString::Initialiser CauseNamesInit[] = {
+    { Q931::UnallocatedNumber,           "Unallocated number"              },
+    { Q931::NoRouteToNetwork,            "No route to network"             },
+    { Q931::NoRouteToDestination,        "No route to destination"         },
+    { Q931::SendSpecialTone,             "Send special tone"               },
+    { Q931::MisdialledTrunkPrefix,       "Misdialled trunk prefix"         },
+    { Q931::ChannelUnacceptable,         "Channel unacceptable"            },
+    { Q931::NormalCallClearing,          "Normal call clearing"            },
+    { Q931::UserBusy,                    "User busy"                       },
+    { Q931::NoResponse,                  "No response"                     },
+    { Q931::NoAnswer,                    "No answer"                       },
+    { Q931::SubscriberAbsent,            "Subscriber absent"               },
+    { Q931::CallRejected,                "Call rejected"                   },
+    { Q931::NumberChanged,               "Number changed"                  },
+    { Q931::Redirection,                 "Redirection"                     },
+    { Q931::ExchangeRoutingError,        "Exchange routing error"          },
+    { Q931::NonSelectedUserClearing,     "Non selected user clearing"      },
+    { Q931::DestinationOutOfOrder,       "Destination out of order"        },
+    { Q931::InvalidNumberFormat,         "Invalid number format"           },
+    { Q931::FacilityRejected,            "Facility rejected"               },
+    { Q931::StatusEnquiryResponse,       "Status enquiry response"         },
+    { Q931::NormalUnspecified,           "Normal unspecified"              },
+    { Q931::NoCircuitChannelAvailable,   "No circuit/channel available"    },
+    { Q931::NetworkOutOfOrder,           "Network out of order"            },
+    { Q931::TemporaryFailure,            "Temporary failure"               },
+    { Q931::Congestion,                  "Congestion"                      },
+    { Q931::RequestedCircuitNotAvailable,"RequestedCircuitNotAvailable"    },
+    { Q931::ResourceUnavailable,         "Resource unavailable"            },
+    { Q931::ServiceOptionNotAvailable,   "Service or option not available" },
+    { Q931::InvalidCallReference,        "Invalid call reference"          },
+    { Q931::IncompatibleDestination,     "Incompatible destination"        },
+    { Q931::IENonExistantOrNotImplemented,"IE non-existent or not implemented" },
+    { Q931::TimerExpiry,                 "Recovery from timer expiry"      },
+    { Q931::ProtocolErrorUnspecified,    "Protocol error, unspecified"     },
+    { Q931::InterworkingUnspecified,     "Interworking, unspecified"       }
+  };
+  static const POrdinalToString CauseNames(PARRAYSIZE(CauseNamesInit), CauseNamesInit);
+
+  if (CauseNames.Contains((PINDEX)cause))
+    strm << CauseNames[cause];
+  else if (cause < Q931::ErrorInCauseIE)
+    strm << "0x" << hex << (unsigned)cause << dec << " (" << (unsigned)cause << ')';
+  else
+    strm << "N/A";
+
+  return strm;
+}
+
+
+///////////////////////////////////////////////////////////////////////////////
+
 Q931::Q931()
 {
   protocolDiscriminator = 8;  // Q931 always has 00001000
@@ -183,11 +322,17 @@ Q931::Q931()
 }
 
 
+Q931::Q931(const Q931 & other)
+{
+  operator=(other);
+}
+
+
 Q931 & Q931::operator=(const Q931 & other)
 {
-  protocolDiscriminator = other.protocolDiscriminator;
   callReference = other.callReference;
   fromDestination = other.fromDestination;
+  protocolDiscriminator = other.protocolDiscriminator;
   messageType = other.messageType;
 
   informationElements.RemoveAll();
@@ -297,12 +442,7 @@ void Q931::BuildStatus(int callRef, BOOL fromDest)
   callReference = callRef;
   fromDestination = fromDest;
   informationElements.RemoveAll();
-  PBYTEArray data(1);
-
-  // Call State as per Q.931 section 4.5.7
-  data[0] = 0; // Use CCITT Null
-  SetIE(CallStateIE, data);
-
+  SetCallState(CallState_Active);
   // Cause field as per Q.850
   SetCause(StatusEnquiryResponse);
 }
@@ -357,19 +497,18 @@ BOOL Q931::Decode(const PBYTEArray & data)
       int len = data[offset++];
 
       if (discriminator == UserUserIE) {
-        // Special case of User-user field, there is some confusion here as
-        // the Q931 documentation claims the length is a single byte,
-        // unfortunately all H.323 based apps have a 16 bit length here, so
-        // we allow for said longer length. There is presumably an addendum
-        // to Q931 which describes this, and provides a means to discriminate
-        // between the old 1 byte and the new 2 byte systems. However, at
-        // present we assume it is always 2 bytes until we find something that
-        // breaks it.
+        // Special case of User-user field. See 7.2.2.31/H.225.0v4.
         len <<= 8;
         len |= data[offset++];
 
         // we also have a protocol discriminator, which we ignore
         offset++;
+
+	// before decrementing the length, make sure it is not zero
+	if (len == 0)
+          return FALSE;
+
+        // adjust for protocol discriminator
         len--;
       }
 
@@ -449,57 +588,47 @@ BOOL Q931::Encode(PBYTEArray & data) const
 }
 
 
-static PString Q931IENameString(int number)
-{
-  switch (number) {
-    case Q931::BearerCapabilityIE :
-      return "Bearer-Capability";
-    case Q931::CauseIE :
-      return "Cause";
-    case Q931::FacilityIE :
-      return "Facility";
-    case Q931::ProgressIndicatorIE :
-      return "Progress-Indicator";
-    case Q931::CallStateIE :
-      return "Call-State";
-    case Q931::DisplayIE :
-      return "Display";
-    case Q931::SignalIE :
-      return "Signal";
-    case Q931::ConnectedNumberIE :
-      return "Connected-Number";
-    case Q931::CallingPartyNumberIE :
-      return "Calling-Party-Number";
-    case Q931::CalledPartyNumberIE :
-      return "Called-Party-Number";
-    case Q931::RedirectingNumberIE :
-      return "Redirecting-Number";
-    case Q931::UserUserIE :
-      return "User-User";
-  }
-  return psprintf("0x%02x", number);
-}
-
-
 void Q931::PrintOn(ostream & strm) const
 {
   int indent = strm.precision() + 2;
+  _Ios_Fmtflags flags = strm.flags();
+
   strm << "{\n"
        << setw(indent+24) << "protocolDiscriminator = " << protocolDiscriminator << '\n'
        << setw(indent+16) << "callReference = " << callReference << '\n'
        << setw(indent+7)  << "from = " << (fromDestination ? "destination" : "originator") << '\n'
        << setw(indent+14) << "messageType = " << GetMessageTypeName() << '\n';
 
-  for (unsigned discriminator = 0; discriminator < 256; discriminator++)
-    if (informationElements.Contains(discriminator))
-      strm << setw(indent+4)
-           << "IE: " << Q931IENameString(discriminator) << " = {\n"
-           << hex << setfill('0')
-           << setprecision(indent+2) << informationElements[discriminator] << '\n'
-           << dec << setfill(' ')
+  for (unsigned discriminator = 0; discriminator < 256; discriminator++) {
+    if (informationElements.Contains(discriminator)) {
+      strm << setw(indent+4) << "IE: " << (InformationElementCodes)discriminator;
+      if (discriminator == CauseIE) {
+        if (informationElements[discriminator].GetSize() > 1)
+          strm << " - " << (CauseValues)(informationElements[discriminator][1]&0x7f);
+      }
+      strm << " = {\n"
+           << hex << setfill('0') << resetiosflags(ios::floatfield)
+           << setprecision(indent+2) << setw(16);
+
+      PBYTEArray value = informationElements[discriminator];
+      if (value.GetSize() <= 32 || (flags&ios::floatfield) != ios::fixed)
+        strm << value;
+      else {
+        PBYTEArray truncatedArray(value, 32);
+        strm << truncatedArray << '\n'
+             << setfill(' ')
+             << setw(indent+5) << "...";
+      }
+
+      strm << dec << setfill(' ')
+           << '\n'
            << setw(indent+2) << "}\n";
+    }
+  }
 
   strm << setw(indent-1) << "}";
+
+  strm.flags(flags);
 }
 
 
@@ -591,37 +720,52 @@ void Q931::SetBearerCapabilities(InformationTransferCapability capability,
                                  unsigned codingStandard,
                                  unsigned userInfoLayer1)
 {
-  PBYTEArray data(3);
+  BYTE data[4];
+  PINDEX size = 1;
   data[0] = (BYTE)(0x80 | ((codingStandard&3) << 5) | (capability&31));
 
-  // Note this is always "Circuit Mode"
-  switch (transferRate) {
-    case 1 :
-      data[1] = 0x90;
+  switch (codingStandard) {
+    case 0 :  // ITU-T standardized coding
+      size = 3;
+
+      // Note this is always "Circuit Mode"
+      switch (transferRate) {
+        case 1 :
+          data[1] = 0x90;
+          break;
+        case 2 :
+          data[1] = 0x91;
+          break;
+        case 6 :
+          data[1] = 0x93;
+          break;
+        case 24 :
+          data[1] = 0x95;
+          break;
+        case 30 :
+          data[1] = 0x97;
+          break;
+        default :
+          PAssert(transferRate > 0 && transferRate < 128, PInvalidParameter);
+          data[1] = 0x18;
+          data[2] = (BYTE)(0x80 | transferRate);
+          size = 4;
+      }
+
+      PAssert(userInfoLayer1 >= 2 && userInfoLayer1 <= 5, PInvalidParameter);
+      data[size-1] = (BYTE)(0x80 | (1<<5) | userInfoLayer1);
       break;
-    case 2 :
-      data[1] = 0x91;
-      break;
-    case 6 :
-      data[1] = 0x93;
-      break;
-    case 24 :
-      data[1] = 0x95;
-      break;
-    case 30 :
-      data[1] = 0x97;
+
+    case 1 : // Other international standard
+      size = 2;
+      data[1] = 0x80; // Call independent signalling connection
       break;
+
     default :
-      PAssert(transferRate > 0 && transferRate < 128, PInvalidParameter);
-      data.SetSize(4);
-      data[1] = 0x18;
-      data[2] = (BYTE)(0x80 | transferRate);
+      break;
   }
 
-  PAssert(userInfoLayer1 >= 2 && userInfoLayer1 <= 5, PInvalidParameter);
-  data[data.GetSize()-1] = (BYTE)(0x80 | (1<<5) | userInfoLayer1);
-
-  SetIE(BearerCapabilityIE, data);
+  SetIE(BearerCapabilityIE, PBYTEArray(data, size));
 }
 
 
@@ -669,7 +813,7 @@ BOOL Q931::GetBearerCapabilities(InformationTransferCapability & capability,
   }
 
   if (userInfoLayer1 != NULL && ((data[nextByte]>>5)&3) == 1)
-    *userInfoLayer1 = data[nextByte]&0x31;
+    *userInfoLayer1 = data[nextByte]&0x1f;
 
   return TRUE;
 }
@@ -709,6 +853,34 @@ Q931::CauseValues Q931::GetCause(unsigned * standard, unsigned * location) const
 }
 
 
+void Q931::SetCallState(CallStates value, unsigned standard)
+{
+  if (value >= CallState_ErrorInIE)
+    return;
+
+  // Call State as per Q.931 section 4.5.7
+  PBYTEArray data(1);
+  data[0] = (BYTE)(((standard&3) << 6) | value);
+  SetIE(CallStateIE, data);
+}
+
+
+Q931::CallStates Q931::GetCallState(unsigned * standard) const
+{
+  if (!HasIE(SignalIE))
+    return CallState_ErrorInIE;
+
+  PBYTEArray data = GetIE(CallStateIE);
+  if (data.IsEmpty())
+    return CallState_ErrorInIE;
+
+  if (standard != NULL)
+    *standard = (data[0] >> 6)&3;
+
+  return (CallStates)(data[0]&0x3f);
+}
+
+
 void Q931::SetSignalInfo(SignalInfo value)
 {
   PBYTEArray data(1);
@@ -730,6 +902,26 @@ Q931::SignalInfo Q931::GetSignalInfo() const
 }
 
 
+void Q931::SetKeypad(const PString & digits)
+{
+  PBYTEArray bytes((const BYTE *)(const char *)digits, digits.GetLength()+1);
+  SetIE(KeypadIE, bytes);
+}
+
+
+PString Q931::GetKeypad() const
+{
+  if (!HasIE(Q931::KeypadIE))
+    return PString();
+
+  PBYTEArray digits = GetIE(Q931::KeypadIE);
+  if (digits.IsEmpty())
+    return PString();
+
+  return PString((const char *)(const BYTE *)digits, digits.GetSize());
+}
+
+
 void Q931::SetProgressIndicator(unsigned description,
                                 unsigned codingStandard,
                                 unsigned location)
@@ -743,7 +935,7 @@ void Q931::SetProgressIndicator(unsigned description,
 
 BOOL Q931::GetProgressIndicator(unsigned & description,
                                 unsigned * codingStandard,
-                                unsigned * location)
+                                unsigned * location) const
 {
   if (!HasIE(ProgressIndicatorIE))
     return FALSE;
@@ -993,4 +1185,103 @@ void Q931::SetConnectedNumber(const PString & number,
 }
 
 
+void Q931::SetChannelIdentification(unsigned interfaceType,
+                                    unsigned preferredOrExclusive,
+                                    int      channelNumber)
+{
+  // Known limitations:
+  //  - the interface identifier cannot be specified
+  //  - channel in PRI can only be indicated by number and cannot be indicated by map
+  //  - one and only one channel can be indicated
+  //  - the coding standard is always ITU Q.931
+
+  PBYTEArray bytes;
+  bytes.SetSize(1);
+
+  PAssert(interfaceType < 2, PInvalidParameter);
+
+  if (interfaceType == 0) { // basic rate
+    if (channelNumber == -1) { // any channel
+      bytes[0] = 0x80 | 0x04 | 0x03;
+    }
+    if (channelNumber == 0) { // D channel
+      bytes[0] = 0x80;
+    }    
+    if (channelNumber > 0) { // B channel
+      bytes[0] = (BYTE)(0x80 | 0x04 | ((preferredOrExclusive & 0x01) << 3) | (channelNumber & 0x03));
+    }
+  }
+
+  if (interfaceType == 1) { // primary rate
+    if (channelNumber == -1) { // any channel
+      bytes[0] = 0x80 | 0x20 | 0x04 | 0x03;
+    }
+    if (channelNumber == 0) { // D channel
+      bytes[0] = 0x80 | 0x20;
+    }    
+    if (channelNumber > 0) { // B channel
+      bytes.SetSize(3);
+
+      bytes[0] = (BYTE)(0x80 | 0x20 | 0x04 | ((preferredOrExclusive & 0x01) << 3) | 0x01);
+      bytes[1] = 0x80 | 0x03;
+      bytes[2] = (BYTE)(0x80 | channelNumber);
+    }
+  }
+
+  SetIE(ChannelIdentificationIE, bytes);
+}
+
+
+BOOL Q931::GetChannelIdentification(unsigned * interfaceType,
+                                    unsigned * preferredOrExclusive,
+                                    int      * channelNumber) const
+{
+  if (!HasIE(ChannelIdentificationIE))
+    return FALSE;
+
+  PBYTEArray bytes = GetIE(ChannelIdentificationIE);
+  if (bytes.GetSize() < 1)
+    return FALSE;
+
+  *interfaceType        = (bytes[0]>>5) & 0x01;
+  *preferredOrExclusive = (bytes[0]>>3) & 0x01;
+
+  if (*interfaceType == 0) {  // basic rate
+    if ( (bytes[0] & 0x04) == 0 ) {  // D Channel
+      *channelNumber = 0;
+    }
+    else {
+      if ( (bytes[0] & 0x03) == 0x03 ) {  // any channel
+        *channelNumber = -1;
+      }
+      else { // B Channel
+        *channelNumber = (bytes[0] & 0x03);
+      }
+    }
+  }
+
+  if (*interfaceType == 1) {  // primary rate
+    if ( (bytes[0] & 0x04) == 0 ) {  // D Channel
+      *channelNumber = 0;
+    }
+    else {
+      if ( (bytes[0] & 0x03) == 0x03 ) {  // any channel
+        *channelNumber = -1;
+      }
+      else { // B Channel
+        if (bytes.GetSize() < 3)
+          return FALSE;
+
+        if (bytes[1] != 0x83)
+          return FALSE;
+
+        *channelNumber = bytes[2] & 0x7f;
+      }
+    }
+  }
+
+  return TRUE;
+}
+
+
 /////////////////////////////////////////////////////////////////////////////
